# Based on the chroot connection plugin by Maykel Moya
#
# (c) 2014, Lorin Hochstein
# (c) 2015, Leendert Brouwer (https://github.com/objectified)
# (c) 2015, Toshio Kuratomi <tkuratomi@ansible.com>
# Copyright (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
author:
  - Lorin Hochestein (!UNKNOWN)
  - Leendert Brouwer (!UNKNOWN)
name: docker
short_description: Run tasks in docker containers
description:
  - Run commands or put/fetch files to an existing docker container.
  - Uses the Docker CLI to execute commands in the container. If you prefer to directly connect to the Docker daemon, use
    the P(community.docker.docker_api#connection) connection plugin.
options:
  remote_addr:
    description:
      - The name of the container you want to access.
    default: inventory_hostname
    vars:
      - name: inventory_hostname
      - name: ansible_host
      - name: ansible_docker_host
  remote_user:
    description:
      - The user to execute as inside the container.
      - If Docker is too old to allow this (< 1.7), the one set by Docker itself will be used.
    vars:
      - name: ansible_user
      - name: ansible_docker_user
    ini:
      - section: defaults
        key: remote_user
    env:
      - name: ANSIBLE_REMOTE_USER
    cli:
      - name: user
    keyword:
      - name: remote_user
  docker_extra_args:
    description:
      - Extra arguments to pass to the docker command line.
    default: ''
    vars:
      - name: ansible_docker_extra_args
    ini:
      - section: docker_connection
        key: extra_cli_args
  container_timeout:
    default: 10
    description:
      - Controls how long we can wait to access reading output from the container once execution started.
    env:
      - name: ANSIBLE_TIMEOUT
      - name: ANSIBLE_DOCKER_TIMEOUT
        version_added: 2.2.0
    ini:
      - key: timeout
        section: defaults
      - key: timeout
        section: docker_connection
        version_added: 2.2.0
    vars:
      - name: ansible_docker_timeout
        version_added: 2.2.0
    cli:
      - name: timeout
    type: integer
  extra_env:
    description:
      - Provide extra environment variables to set when running commands in the Docker container.
      - This option can currently only be provided as Ansible variables due to limitations of ansible-core's configuration
        manager.
    vars:
      - name: ansible_docker_extra_env
    type: dict
    version_added: 3.12.0
  working_dir:
    description:
      - The directory inside the container to run commands in.
      - Requires Docker CLI version 18.06 or later.
    env:
      - name: ANSIBLE_DOCKER_WORKING_DIR
    ini:
      - key: working_dir
        section: docker_connection
    vars:
      - name: ansible_docker_working_dir
    type: string
    version_added: 3.12.0
  privileged:
    description:
      - Whether commands should be run with extended privileges.
      - B(Note) that this allows command to potentially break out of the container. Use with care!
    env:
      - name: ANSIBLE_DOCKER_PRIVILEGED
    ini:
      - key: privileged
        section: docker_connection
    vars:
      - name: ansible_docker_privileged
    type: boolean
    default: false
    version_added: 3.12.0
"""

import fcntl
import os
import os.path
import re
import selectors
import subprocess
import typing as t
from shlex import quote

from ansible.errors import AnsibleConnectionFailure, AnsibleError, AnsibleFileNotFound
from ansible.module_utils.common.process import get_bin_path
from ansible.module_utils.common.text.converters import to_bytes, to_text
from ansible.plugins.connection import BUFSIZE, ConnectionBase
from ansible.utils.display import Display

from ansible_collections.community.docker.plugins.module_utils._version import (
    LooseVersion,
)

display = Display()


class Connection(ConnectionBase):
    """Local docker based connections"""

    transport = "community.docker.docker"
    has_pipelining = True

    def __init__(self, *args: t.Any, **kwargs: t.Any) -> None:
        super().__init__(*args, **kwargs)

        # Note: docker supports running as non-root in some configurations.
        # (For instance, setting the UNIX socket file to be readable and
        # writable by a specific UNIX group and then putting users into that
        # group).  Therefore we do not check that the user is root when using
        # this connection.  But if the user is getting a permission denied
        # error it probably means that docker on their system is only
        # configured to be connected to by root and they are not running as
        # root.

        self._docker_args: list[bytes | str] = []
        self._container_user_cache: dict[str, str | None] = {}
        self._version: str | None = None
        self.remote_user: str | None = None
        self.timeout: int | float | None = None

        # Windows uses Powershell modules
        if getattr(self._shell, "_IS_WINDOWS", False):
            self.module_implementation_preferences = (".ps1", ".exe", "")

        if "docker_command" in kwargs:
            self.docker_cmd = kwargs["docker_command"]
        else:
            try:
                self.docker_cmd = get_bin_path("docker")
            except ValueError as exc:
                raise AnsibleError("docker command not found in PATH") from exc

    @staticmethod
    def _sanitize_version(version: str) -> str:
        version = re.sub("[^0-9a-zA-Z.]", "", version)
        version = re.sub("^v", "", version)
        return version

    def _old_docker_version(self) -> tuple[list[str], str, bytes, int]:
        cmd_args = self._docker_args

        old_version_subcommand = ["version"]

        old_docker_cmd = [self.docker_cmd] + cmd_args + old_version_subcommand
        with subprocess.Popen(
            old_docker_cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE
        ) as p:
            cmd_output, err = p.communicate()

        return old_docker_cmd, to_text(cmd_output), err, p.returncode

    def _new_docker_version(self) -> tuple[list[str], str, bytes, int]:
        # no result yet, must be newer Docker version
        cmd_args = self._docker_args

        new_version_subcommand = ["version", "--format", "'{{.Server.Version}}'"]

        new_docker_cmd = [self.docker_cmd] + cmd_args + new_version_subcommand
        with subprocess.Popen(
            new_docker_cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE
        ) as p:
            cmd_output, err = p.communicate()
            return new_docker_cmd, to_text(cmd_output), err, p.returncode

    def _get_docker_version(self) -> str:
        cmd, cmd_output, err, returncode = self._old_docker_version()
        if returncode == 0:
            for line in to_text(cmd_output, errors="surrogate_or_strict").split("\n"):
                if line.startswith("Server version:"):  # old docker versions
                    return self._sanitize_version(line.split()[2])

        cmd, cmd_output, err, returncode = self._new_docker_version()
        if returncode:
            raise AnsibleError(
                f"Docker version check ({to_text(cmd)}) failed: {to_text(err)}"
            )

        return self._sanitize_version(to_text(cmd_output, errors="surrogate_or_strict"))

    def _get_docker_remote_user(self) -> str | None:
        """Get the default user configured in the docker container"""
        container = self.get_option("remote_addr")
        if container in self._container_user_cache:
            return self._container_user_cache[container]
        with subprocess.Popen(
            [self.docker_cmd, "inspect", "--format", "{{.Config.User}}", container],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        ) as p:
            out_b, err_b = p.communicate()
            out = to_text(out_b, errors="surrogate_or_strict")

            if p.returncode != 0:
                display.warning(
                    f"unable to retrieve default user from docker container: {out} {to_text(err_b)}"
                )
                self._container_user_cache[container] = None
                return None

        # The default exec user is root, unless it was changed in the Dockerfile with USER
        user = out.strip() or "root"
        self._container_user_cache[container] = user
        return user

    def _build_exec_cmd(self, cmd: list[bytes | str]) -> list[bytes | str]:
        """Build the local docker exec command to run cmd on remote_host

        If remote_user is available and is supported by the docker
        version we are using, it will be provided to docker exec.
        """

        local_cmd = [self.docker_cmd]

        if self._docker_args:
            local_cmd += self._docker_args

        local_cmd += [b"exec"]

        if self.remote_user is not None:
            local_cmd += [b"-u", self.remote_user]

        if self.get_option("extra_env"):
            for k, v in self.get_option("extra_env").items():
                for val, what in ((k, "Key"), (v, "Value")):
                    if not isinstance(val, str):
                        raise AnsibleConnectionFailure(
                            f"Non-string {what.lower()} found for extra_env option. Ambiguous env options must be "
                            "wrapped in quotes to avoid them being interpreted when directly specified "
                            "in YAML, or explicitly converted to strings when the option is templated. "
                            f"{what}: {val!r}"
                        )
                local_cmd += [
                    b"-e",
                    b"%s=%s"
                    % (
                        to_bytes(k, errors="surrogate_or_strict"),
                        to_bytes(v, errors="surrogate_or_strict"),
                    ),
                ]

        if self.get_option("working_dir") is not None:
            local_cmd += [
                b"-w",
                to_bytes(self.get_option("working_dir"), errors="surrogate_or_strict"),
            ]
            if self.docker_version != "dev" and LooseVersion(
                self.docker_version
            ) < LooseVersion("18.06"):
                # https://github.com/docker/cli/pull/732, first appeared in release 18.06.0
                raise AnsibleConnectionFailure(
                    f"Providing the working directory requires Docker CLI version 18.06 or newer. You have Docker CLI version {self.docker_version}."
                )

        if self.get_option("privileged"):
            local_cmd += [b"--privileged"]

        # -i is needed to keep stdin open which allows pipelining to work
        local_cmd += [b"-i", self.get_option("remote_addr")] + cmd

        return local_cmd

    def _set_docker_args(self) -> None:
        # TODO: this is mostly for backwards compatibility, play_context is used as fallback for older versions
        # docker arguments
        del self._docker_args[:]
        extra_args = self.get_option("docker_extra_args") or getattr(
            self._play_context, "docker_extra_args", ""
        )
        if extra_args:
            self._docker_args += extra_args.split(" ")

    def _set_conn_data(self) -> None:
        """initialize for the connection, cannot do only in init since all data is not ready at that point"""

        self._set_docker_args()

        self.remote_user = self.get_option("remote_user")
        if self.remote_user is None and self._play_context.remote_user is not None:
            self.remote_user = self._play_context.remote_user

        # timeout, use unless default and pc is different, backwards compat
        self.timeout = self.get_option("container_timeout")
        if self.timeout == 10 and self.timeout != self._play_context.timeout:
            self.timeout = self._play_context.timeout

    @property
    def docker_version(self) -> str:
        if not self._version:
            self._set_docker_args()

            self._version = self._get_docker_version()
            if self._version == "dev":
                display.warning(
                    'Docker version number is "dev". Will assume latest version.'
                )
            if self._version != "dev" and LooseVersion(self._version) < LooseVersion(
                "1.3"
            ):
                raise AnsibleError(
                    "docker connection type requires docker 1.3 or higher"
                )
        return self._version

    def _get_actual_user(self) -> str | None:
        if self.remote_user is not None:
            # An explicit user is provided
            if self.docker_version == "dev" or LooseVersion(
                self.docker_version
            ) >= LooseVersion("1.7"):
                # Support for specifying the exec user was added in docker 1.7
                return self.remote_user
            self.remote_user = None
            actual_user = self._get_docker_remote_user()
            if actual_user != self.get_option("remote_user"):
                display.warning(
                    f"docker {self.docker_version} does not support remote_user, using container default: {actual_user or '?'}"
                )
            return actual_user
        if self._display.verbosity > 2:
            # Since we are not setting the actual_user, look it up so we have it for logging later
            # Only do this if display verbosity is high enough that we'll need the value
            # This saves overhead from calling into docker when we do not need to.
            return self._get_docker_remote_user()
        return None

    def _connect(self) -> t.Self:
        """Connect to the container. Nothing to do"""
        super()._connect()  # type: ignore[safe-super]
        if not self._connected:
            self._set_conn_data()
            actual_user = self._get_actual_user()
            display.vvv(
                f"ESTABLISH DOCKER CONNECTION FOR USER: {actual_user or '?'}",
                host=self.get_option("remote_addr"),
            )
            self._connected = True
        return self

    def exec_command(
        self, cmd: str, in_data: bytes | None = None, sudoable: bool = False
    ) -> tuple[int, bytes, bytes]:
        """Run a command on the docker host"""

        self._set_conn_data()

        super().exec_command(cmd, in_data=in_data, sudoable=sudoable)  # type: ignore[safe-super]

        local_cmd = self._build_exec_cmd([self._play_context.executable, "-c", cmd])

        display.vvv(f"EXEC {to_text(local_cmd)}", host=self.get_option("remote_addr"))
        display.debug("opening command with Popen()")

        local_cmd = [to_bytes(i, errors="surrogate_or_strict") for i in local_cmd]

        with subprocess.Popen(
            local_cmd,
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        ) as p:
            assert p.stdin is not None
            assert p.stdout is not None
            assert p.stderr is not None
            display.debug("done running command with Popen()")

            if self.become and self.become.expect_prompt() and sudoable:
                fcntl.fcntl(
                    p.stdout,
                    fcntl.F_SETFL,
                    fcntl.fcntl(p.stdout, fcntl.F_GETFL) | os.O_NONBLOCK,
                )
                fcntl.fcntl(
                    p.stderr,
                    fcntl.F_SETFL,
                    fcntl.fcntl(p.stderr, fcntl.F_GETFL) | os.O_NONBLOCK,
                )
                selector = selectors.DefaultSelector()
                selector.register(p.stdout, selectors.EVENT_READ)
                selector.register(p.stderr, selectors.EVENT_READ)

                become_output = b""
                try:
                    while not self.become.check_success(
                        become_output
                    ) and not self.become.check_password_prompt(become_output):
                        events = selector.select(self.timeout)
                        if not events:
                            stdout, stderr = p.communicate()
                            raise AnsibleError(
                                "timeout waiting for privilege escalation password prompt:\n"
                                + to_text(become_output)
                            )

                        chunks = b""
                        for key, dummy_event in events:
                            if key.fileobj == p.stdout:
                                chunk = p.stdout.read()
                                if chunk:
                                    chunks += chunk
                            elif key.fileobj == p.stderr:
                                chunk = p.stderr.read()
                                if chunk:
                                    chunks += chunk

                        if not chunks:
                            stdout, stderr = p.communicate()
                            raise AnsibleError(
                                "privilege output closed while waiting for password prompt:\n"
                                + to_text(become_output)
                            )
                        become_output += chunks
                finally:
                    selector.close()

                if not self.become.check_success(become_output):
                    become_pass = self.become.get_option(
                        "become_pass", playcontext=self._play_context
                    )
                    p.stdin.write(
                        to_bytes(become_pass, errors="surrogate_or_strict") + b"\n"
                    )
                fcntl.fcntl(
                    p.stdout,
                    fcntl.F_SETFL,
                    fcntl.fcntl(p.stdout, fcntl.F_GETFL) & ~os.O_NONBLOCK,
                )
                fcntl.fcntl(
                    p.stderr,
                    fcntl.F_SETFL,
                    fcntl.fcntl(p.stderr, fcntl.F_GETFL) & ~os.O_NONBLOCK,
                )

            display.debug("getting output with communicate()")
            stdout, stderr = p.communicate(in_data)
            display.debug("done communicating")

            display.debug("done with docker.exec_command()")
            return (p.returncode, stdout, stderr)

    def _prefix_login_path(self, remote_path: str) -> str:
        """Make sure that we put files into a standard path

        If a path is relative, then we need to choose where to put it.
        ssh chooses $HOME but we are not guaranteed that a home dir will
        exist in any given chroot.  So for now we are choosing "/" instead.
        This also happens to be the former default.

        Can revisit using $HOME instead if it is a problem
        """
        if getattr(self._shell, "_IS_WINDOWS", False):
            import ntpath

            return ntpath.normpath(remote_path)
        if not remote_path.startswith(os.path.sep):
            remote_path = os.path.join(os.path.sep, remote_path)
        return os.path.normpath(remote_path)

    def put_file(self, in_path: str, out_path: str) -> None:
        """Transfer a file from local to docker container"""
        self._set_conn_data()
        super().put_file(in_path, out_path)  # type: ignore[safe-super]
        display.vvv(f"PUT {in_path} TO {out_path}", host=self.get_option("remote_addr"))

        out_path = self._prefix_login_path(out_path)
        if not os.path.exists(to_bytes(in_path, errors="surrogate_or_strict")):
            raise AnsibleFileNotFound(
                f"file or module does not exist: {to_text(in_path)}"
            )

        out_path = quote(out_path)
        # Older docker does not have native support for copying files into
        # running containers, so we use docker exec to implement this
        # Although docker version 1.8 and later provide support, the
        # owner and group of the files are always set to root
        with open(to_bytes(in_path, errors="surrogate_or_strict"), "rb") as in_file:
            if not os.fstat(in_file.fileno()).st_size:
                count = " count=0"
            else:
                count = ""
            args = self._build_exec_cmd(
                [
                    self._play_context.executable,
                    "-c",
                    f"dd of={out_path} bs={BUFSIZE}{count}",
                ]
            )
            args = [to_bytes(i, errors="surrogate_or_strict") for i in args]
            try:
                # pylint: disable-next=consider-using-with
                p = subprocess.Popen(
                    args, stdin=in_file, stdout=subprocess.PIPE, stderr=subprocess.PIPE
                )
            except OSError as exc:
                raise AnsibleError(
                    "docker connection requires dd command in the container to put files"
                ) from exc
            stdout, stderr = p.communicate()

            if p.returncode != 0:
                raise AnsibleError(
                    f"failed to transfer file {to_text(in_path)} to {to_text(out_path)}:\n{to_text(stdout)}\n{to_text(stderr)}"
                )

    def fetch_file(self, in_path: str, out_path: str) -> None:
        """Fetch a file from container to local."""
        self._set_conn_data()
        super().fetch_file(in_path, out_path)  # type: ignore[safe-super]
        display.vvv(
            f"FETCH {in_path} TO {out_path}", host=self.get_option("remote_addr")
        )

        in_path = self._prefix_login_path(in_path)
        # out_path is the final file path, but docker takes a directory, not a
        # file path
        out_dir = os.path.dirname(out_path)

        args = [
            self.docker_cmd,
            "cp",
            f"{self.get_option('remote_addr')}:{in_path}",
            out_dir,
        ]
        args = [to_bytes(i, errors="surrogate_or_strict") for i in args]

        with subprocess.Popen(
            args, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE
        ) as p:
            p.communicate()

            if getattr(self._shell, "_IS_WINDOWS", False):
                import ntpath

                actual_out_path = ntpath.join(out_dir, ntpath.basename(in_path))
            else:
                actual_out_path = os.path.join(out_dir, os.path.basename(in_path))

            if p.returncode != 0:
                # Older docker does not have native support for fetching files command `cp`
                # If `cp` fails, try to use `dd` instead
                args = self._build_exec_cmd(
                    [
                        self._play_context.executable,
                        "-c",
                        f"dd if={in_path} bs={BUFSIZE}",
                    ]
                )
                args = [to_bytes(i, errors="surrogate_or_strict") for i in args]
                with open(
                    to_bytes(actual_out_path, errors="surrogate_or_strict"), "wb"
                ) as out_file:
                    try:
                        # pylint: disable-next=consider-using-with
                        pp = subprocess.Popen(
                            args,
                            stdin=subprocess.PIPE,
                            stdout=out_file,
                            stderr=subprocess.PIPE,
                        )
                    except OSError as exc:
                        raise AnsibleError(
                            "docker connection requires dd command in the container to put files"
                        ) from exc
                    stdout, stderr = pp.communicate()

                    if pp.returncode != 0:
                        raise AnsibleError(
                            f"failed to fetch file {in_path} to {out_path}:\n{stdout!r}\n{stderr!r}"
                        )

        # Rename if needed
        if actual_out_path != out_path:
            os.rename(
                to_bytes(actual_out_path, errors="strict"),
                to_bytes(out_path, errors="strict"),
            )

    def close(self) -> None:
        """Terminate the connection. Nothing to do for Docker"""
        super().close()  # type: ignore[safe-super]
        self._connected = False

    def reset(self) -> None:
        # Clear container user cache
        self._container_user_cache = {}



#!/usr/bin/python

# Copyright (c) 2018, Bruce Smith <Bruce.Smith.IT@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: nictagadm
short_description: Manage nic tags on SmartOS systems
description:
  - Create or delete nic tags on SmartOS systems.
author:
  - Bruce Smith (@SmithX10)
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  name:
    description:
      - Name of the nic tag.
    required: true
    type: str
  mac:
    description:
      - Specifies the O(mac) address to attach the nic tag to when not creating an O(etherstub).
      - Parameters O(mac) and O(etherstub) are mutually exclusive.
    type: str
  etherstub:
    description:
      - Specifies that the nic tag is attached to a created O(etherstub).
      - Parameter O(etherstub) is mutually exclusive with both O(mtu), and O(mac).
    type: bool
    default: false
  mtu:
    description:
      - Specifies the size of the O(mtu) of the desired nic tag.
      - Parameters O(mtu) and O(etherstub) are mutually exclusive.
    type: int
  force:
    description:
      - When O(state=absent) this switch uses the C(-f) parameter and delete the nic tag regardless of existing VMs.
    type: bool
    default: false
  state:
    description:
      - Create or delete a SmartOS nic tag.
    type: str
    choices: [absent, present]
    default: present
"""

EXAMPLES = r"""
- name: Create 'storage0' on '00:1b:21:a3:f5:4d'
  community.general.nictagadm:
    name: storage0
    mac: 00:1b:21:a3:f5:4d
    mtu: 9000
    state: present

- name: Remove 'storage0' nic tag
  community.general.nictagadm:
    name: storage0
    state: absent
"""

RETURN = r"""
name:
  description: Nic tag name.
  returned: always
  type: str
  sample: storage0
mac:
  description: MAC Address that the nic tag was attached to.
  returned: always
  type: str
  sample: 00:1b:21:a3:f5:4d
etherstub:
  description: Specifies if the nic tag was created and attached to an etherstub.
  returned: always
  type: bool
  sample: false
mtu:
  description: Specifies which MTU size was passed during the nictagadm add command. mtu and etherstub are mutually exclusive.
  returned: always
  type: int
  sample: 1500
force:
  description: Shows if C(-f) was used during the deletion of a nic tag.
  returned: always
  type: bool
  sample: false
state:
  description: State of the target.
  returned: always
  type: str
  sample: present
"""

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.common.network import is_mac


class NicTag:
    def __init__(self, module):
        self.module = module

        self.name = module.params["name"]
        self.mac = module.params["mac"]
        self.etherstub = module.params["etherstub"]
        self.mtu = module.params["mtu"]
        self.force = module.params["force"]
        self.state = module.params["state"]

        self.nictagadm_bin = self.module.get_bin_path("nictagadm", True)

    def is_valid_mac(self):
        return is_mac(self.mac.lower())

    def nictag_exists(self):
        cmd = [self.nictagadm_bin, "exists", self.name]
        (rc, dummy, dummy) = self.module.run_command(cmd)

        return rc == 0

    def add_nictag(self):
        cmd = [self.nictagadm_bin, "-v", "add"]

        if self.etherstub:
            cmd.append("-l")

        if self.mtu:
            cmd.append("-p")
            cmd.append(f"mtu={self.mtu}")

        if self.mac:
            cmd.append("-p")
            cmd.append(f"mac={self.mac}")

        cmd.append(self.name)

        return self.module.run_command(cmd)

    def delete_nictag(self):
        cmd = [self.nictagadm_bin, "-v", "delete"]

        if self.force:
            cmd.append("-f")

        cmd.append(self.name)

        return self.module.run_command(cmd)


def main():
    module = AnsibleModule(
        argument_spec=dict(
            name=dict(type="str", required=True),
            mac=dict(type="str"),
            etherstub=dict(type="bool", default=False),
            mtu=dict(type="int"),
            force=dict(type="bool", default=False),
            state=dict(type="str", default="present", choices=["absent", "present"]),
        ),
        mutually_exclusive=[
            ["etherstub", "mac"],
            ["etherstub", "mtu"],
        ],
        required_if=[
            ["etherstub", False, ["name", "mac"]],
            ["state", "absent", ["name", "force"]],
        ],
        supports_check_mode=True,
    )

    nictag = NicTag(module)

    rc = None
    out = ""
    err = ""
    result = dict(
        changed=False,
        etherstub=nictag.etherstub,
        force=nictag.force,
        name=nictag.name,
        mac=nictag.mac,
        mtu=nictag.mtu,
        state=nictag.state,
    )

    if not nictag.etherstub and not nictag.is_valid_mac():
        module.fail_json(msg="Invalid MAC Address Value", name=nictag.name, mac=nictag.mac, etherstub=nictag.etherstub)

    if nictag.state == "absent":
        if nictag.nictag_exists():
            if module.check_mode:
                module.exit_json(changed=True)
            (rc, out, err) = nictag.delete_nictag()
            if rc != 0:
                module.fail_json(name=nictag.name, msg=err, rc=rc)
    elif nictag.state == "present":
        if not nictag.nictag_exists():
            if module.check_mode:
                module.exit_json(changed=True)
            (rc, out, err) = nictag.add_nictag()
            if rc is not None and rc != 0:
                module.fail_json(name=nictag.name, msg=err, rc=rc)

    if rc is not None:
        result["changed"] = True
    if out:
        result["stdout"] = out
    if err:
        result["stderr"] = err

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import (absolute_import, division, print_function)

__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = """
---
module: cp_mgmt_interface
short_description: Manages interface objects on Checkpoint over Web Services API
description:
  - Manages interface objects on Checkpoint devices including creating, updating and removing objects.
  - All operations are performed over Web Services API.
  - Available from R82 management version.
version_added: "6.2.0"
author: "Eden Brillant (@chkp-edenbr)"
options:
  name:
    description:
      - Network interface name.
    type: str
    required: True
  gateway_uid:
    description:
      - Gateway or cluster object uid that the interface belongs to.
    type: str
    required: True
  anti_spoofing:
    description:
      - Enable anti-spoofing.
    type: bool
  anti_spoofing_settings:
    description:
      - Anti Spoofing Settings.
    type: dict
    suboptions:
      action:
        description:
          - If packets will be rejected (the Prevent option) or whether the packets will be monitored (the Detect option).
        type: str
        choices: ['prevent', 'detect']
      exclude_packets:
        description:
          - Don't check packets from excluded network.
        type: bool
      excluded_network_name:
        description:
          - Excluded network name.
        type: str
      excluded_network_uid:
        description:
          - Excluded network UID.
        type: str
      spoof_tracking:
        description:
          - Spoof tracking.
        type: str
        choices: ['none', 'log', 'alert']
  cluster_members:
    description:
      - Network interface settings for cluster members.
    type: list
    elements: dict
    suboptions:
      name:
        description:
          - Cluster member network interface name.
        type: str
      member_name:
        description:
          - Cluster member object name.
        type: str
      ip_address:
        description:
          - IPv4 or IPv6 address. If both addresses are required use ipv4-address and ipv6-address fields explicitly.
        type: str
      ipv4_address:
        description:
          - IPv4 address.
        type: str
      ipv6_address:
        description:
          - IPv6 address.
        type: str
      network_mask:
        description:
          - IPv4 or IPv6 network mask. If both masks are required use ipv4-network-mask and ipv6-network-mask fields explicitly. Instead of
            providing mask itself it is possible to specify IPv4 or IPv6 mask length in mask-length field. If both masks length are required use
            ipv4-mask-length and  ipv6-mask-length fields explicitly.
        type: str
      ipv4_network_mask:
        description:
          - IPv4 network address.
        type: str
      ipv6_network_mask:
        description:
          - IPv6 network address.
        type: str
      mask_length:
        description:
          - IPv4 or IPv6 network mask length.
        type: str
      ipv4_mask_length:
        description:
          - IPv4 network mask length.
        type: str
      ipv6_mask_length:
        description:
          - IPv6 network mask length.
        type: str
      tags:
        description:
          - Collection of tag identifiers.
        type: list
        elements: str
      color:
        description:
          - Color of the object. Should be one of existing colors.
        type: str
        choices: ['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green', 'khaki', 'orchid', 'dark orange',
                 'dark sea green', 'pink', 'turquoise', 'dark blue', 'firebrick', 'brown', 'forest green', 'gold', 'dark gold', 'gray', 'dark gray',
                 'light green', 'lemon chiffon', 'coral', 'sea green', 'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive',
                 'orange', 'red', 'sienna', 'yellow']
      comments:
        description:
          - Comments string.
        type: str
      details_level:
        description:
          - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
            representation of the object.
        type: str
        choices: ['uid', 'standard', 'full']
      ignore_warnings:
        description:
          - Apply changes ignoring warnings.
        type: bool
      ignore_errors:
        description:
          - Apply changes ignoring errors. You won't be able to publish such a changes. If ignore-warnings flag was omitted - warnings will also be ignored.
        type: bool
  cluster_network_type:
    description:
      - Cluster interface type.
    type: str
    choices: ['cluster', 'sync', 'cluster + sync', 'private']
  dynamic_ip:
    description:
      - Enable dynamic interface.
    type: bool
  ipv4_address:
    description:
      - IPv4 network address.
    type: str
  ipv4_mask_length:
    description:
      - IPv4 mask length.
    type: int
  ipv4_network_mask:
    description:
      - IPv4 network mask.
    type: str
  ipv6_address:
    description:
      - IPv6 address.
    type: str
  ipv6_mask_length:
    description:
      - IPv6 mask length.
    type: int
  ipv6_network_mask:
    description:
      - IPv6 network mask.
    type: str
  monitored_by_cluster:
    description:
      - When Private is selected as the Cluster interface type, cluster can monitor or not monitor the interface.
    type: bool
  network_interface_type:
    description:
      - Network Interface Type.
    type: str
    choices: ['alias', 'bond', 'bridge', 'bridge member', 'ethernet', 'loopback', '6 in 4 tunnel', 'pppoe', 'vpn tunnel', 'vlan']
  security_zone_settings:
    description:
      - Security Zone Settings.
    type: dict
    suboptions:
      auto_calculated:
        description:
          - Security Zone is calculated according to where the interface leads to.
        type: bool
      specific_zone:
        description:
          - Security Zone specified manually.
        type: str
      specific_security_zone_enabled:
        description:
          - Indicates whether to enable specific security zone.
        type: bool
        version_added: "6.8.0"
  tags:
    description:
      - Collection of tag identifiers.
    type: list
    elements: str
  topology:
    description:
      - Topology configuration.
    type: str
    choices: ['automatic', 'external', 'internal']
  topology_settings:
    description:
      - Topology Settings.
    type: dict
    suboptions:
      interface_leads_to_dmz:
        description:
          - Whether this interface leads to demilitarized zone (perimeter network).
        type: bool
      ip_address_behind_this_interface:
        description:
          - Network settings behind this interface.
        type: str
        choices: ['not defined', 'network defined by the interface ip and net mask', 'network defined by routing', 'specific']
      specific_network:
        description:
          - Network behind this interface.
        type: str
      specific_network_uid:
        description:
          - N/A
        type: str
  color:
    description:
      - Color of the object. Should be one of existing colors.
    type: str
    choices: ['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green', 'khaki', 'orchid', 'dark orange', 'dark sea green',
             'pink', 'turquoise', 'dark blue', 'firebrick', 'brown', 'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon',
             'coral', 'sea green', 'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna', 'yellow']
  comments:
    description:
      - Comments string.
    type: str
  details_level:
    description:
      - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
        representation of the object.
    type: str
    choices: ['uid', 'standard', 'full']
  domains_to_process:
    description:
      - Indicates which domains to process the commands on. It cannot be used with the details-level full, must be run from the System Domain only and
        with ignore-warnings true. Valid values are, CURRENT_DOMAIN, ALL_DOMAINS_ON_THIS_SERVER.
    type: list
    elements: str
  ignore_warnings:
    description:
      - Apply changes ignoring warnings.
    type: bool
  ignore_errors:
    description:
      - Apply changes ignoring errors. You won't be able to publish such a changes. If ignore-warnings flag was omitted - warnings will also be ignored.
    type: bool
extends_documentation_fragment: check_point.mgmt.checkpoint_objects
"""

EXAMPLES = """
- name: add-interface
  cp_mgmt_interface:
    anti_spoofing: true
    anti_spoofing_settings:
      action: detect
      exclude_packets: false
      spoof_tracking: log
    cluster_members:
      - ipv4_address: 2.2.2.1
        ipv4_mask_length: 24
        ipv4_network_mask: 255.255.255.0
        member_name: member1
        name: eth4
      - ipv4_address: 2.2.2.2
        ipv4_mask_length: 24
        ipv4_network_mask: 255.255.255.0
        member_name: member2
        name: eth4
    cluster_network_type: cluster
    gateway_uid: 20ec49e8-8cd8-4ad4-b204-0de8ae4e0e17
    ignore_warnings: false
    ipv4_address: 1.1.1.111
    ipv4_mask_length: 24
    name: eth0
    security_zone_settings:
      auto_calculated: false
      specific_zone: InternalZone
    state: present
    topology: internal
    topology_settings:
      interface_leads_to_dmz: false
      ip_address_behind_this_interface: network defined by routing

- name: set-interface
  cp_mgmt_interface:
    cluster_members:
      - ipv4_address: 4.4.4.1
        ipv4_mask_length: 22
        member_name: memberReal1
        uid: db4f8a63-5a94-46d8-b9e0-a63870bded3d
      - ipv4_address: 4.4.4.2
        ipv4_mask_length: 22
        member_name: memberReal2
        uid: baca571e-8ada-4be9-8966-145388f8e238
    cluster_network_type: cluster + sync
    ipv4_address: 4.4.4.111
    ipv4_mask_length: 22
    state: present
    topology: internal
    topology_settings:
      ip_address_behind_this_interface: network defined by routing
    name: eth0
    gateway_uid: 20ec49e8-8cd8-4ad4-b204-0de8ae4e0e17

- name: delete-interface
  cp_mgmt_interface:
    state: absent
    name: eth0
    gateway_uid: 20ec49e8-8cd8-4ad4-b204-0de8ae4e0e17
"""

RETURN = """
cp_mgmt_interface:
  description: The checkpoint object created or updated.
  returned: always, except when deleting the object.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import checkpoint_argument_spec_for_objects, api_call


def main():
    argument_spec = dict(
        name=dict(type='str', required=True),
        gateway_uid=dict(type='str', required=True),
        anti_spoofing=dict(type='bool'),
        anti_spoofing_settings=dict(type='dict', options=dict(
            action=dict(type='str', choices=['prevent', 'detect']),
            exclude_packets=dict(type='bool'),
            excluded_network_name=dict(type='str'),
            excluded_network_uid=dict(type='str'),
            spoof_tracking=dict(type='str', choices=['none', 'log', 'alert'])
        )),
        cluster_members=dict(type='list', elements='dict', options=dict(
            name=dict(type='str'),
            member_name=dict(type='str'),
            ip_address=dict(type='str'),
            ipv4_address=dict(type='str'),
            ipv6_address=dict(type='str'),
            network_mask=dict(type='str'),
            ipv4_network_mask=dict(type='str'),
            ipv6_network_mask=dict(type='str'),
            mask_length=dict(type='str'),
            ipv4_mask_length=dict(type='str'),
            ipv6_mask_length=dict(type='str'),
            tags=dict(type='list', elements='str'),
            color=dict(type='str', choices=['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan',
                                            'dark green', 'khaki', 'orchid', 'dark orange', 'dark sea green', 'pink', 'turquoise', 'dark blue', 'firebrick',
                                            'brown', 'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon', 'coral',
                                            'sea green', 'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red',
                                            'sienna', 'yellow']),
            comments=dict(type='str'),
            details_level=dict(type='str', choices=['uid', 'standard', 'full']),
            ignore_warnings=dict(type='bool'),
            ignore_errors=dict(type='bool')
        )),
        cluster_network_type=dict(type='str', choices=['cluster', 'sync', 'cluster + sync', 'private']),
        dynamic_ip=dict(type='bool'),
        ipv4_address=dict(type='str'),
        ipv4_mask_length=dict(type='int'),
        ipv4_network_mask=dict(type='str'),
        ipv6_address=dict(type='str'),
        ipv6_mask_length=dict(type='int'),
        ipv6_network_mask=dict(type='str'),
        monitored_by_cluster=dict(type='bool'),
        network_interface_type=dict(type='str', choices=['alias', 'bond', 'bridge', 'bridge member',
                                                         'ethernet', 'loopback', '6 in 4 tunnel', 'pppoe', 'vpn tunnel', 'vlan']),
        security_zone_settings=dict(type='dict', options=dict(
            auto_calculated=dict(type='bool'),
            specific_zone=dict(type='str'),
            specific_security_zone_enabled=dict(type='bool')
        )),
        tags=dict(type='list', elements='str'),
        topology=dict(type='str', choices=['automatic', 'external', 'internal']),
        topology_settings=dict(type='dict', options=dict(
            interface_leads_to_dmz=dict(type='bool'),
            ip_address_behind_this_interface=dict(type='str', choices=['not defined',
                                                                       'network defined by the interface ip and net mask', 'network defined by routing',
                                                                       'specific']),
            specific_network=dict(type='str'),
            specific_network_uid=dict(type='str')
        )),
        color=dict(type='str', choices=['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green',
                                        'khaki', 'orchid', 'dark orange', 'dark sea green', 'pink', 'turquoise', 'dark blue', 'firebrick', 'brown',
                                        'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon', 'coral', 'sea green',
                                        'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna',
                                        'yellow']),
        comments=dict(type='str'),
        details_level=dict(type='str', choices=['uid', 'standard', 'full']),
        domains_to_process=dict(type='list', elements='str'),
        ignore_warnings=dict(type='bool'),
        ignore_errors=dict(type='bool')
    )
    argument_spec.update(checkpoint_argument_spec_for_objects)

    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)
    api_call_object = 'interface'

    result = api_call(module, api_call_object)
    module.exit_json(**result)


if __name__ == '__main__':
    main()



#!/usr/bin/python

# Copyright (c) 2013, 2014, Jan-Piet Mens <jpmens () gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: mqtt
short_description: Publish a message on an MQTT topic for the IoT
description:
  - Publish a message on an MQTT topic.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  server:
    type: str
    description:
      - MQTT broker address/name.
    default: localhost
  port:
    type: int
    description:
      - MQTT broker port number.
    default: 1883
  username:
    type: str
    description:
      - Username to authenticate against the broker.
  password:
    type: str
    description:
      - Password for O(username) to authenticate against the broker.
  client_id:
    type: str
    description:
      - MQTT client identifier.
      - If not specified, it uses a value C(hostname + pid).
  topic:
    type: str
    description:
      - MQTT topic name.
    required: true
  payload:
    type: str
    description:
      - Payload. The special string V("None") may be used to send a NULL (that is, empty) payload which is useful to simply
        notify with the O(topic) or to clear previously retained messages.
    required: true
  qos:
    type: str
    description:
      - QoS (Quality of Service).
    default: "0"
    choices: ["0", "1", "2"]
  retain:
    description:
      - Setting this flag causes the broker to retain (in other words keep) the message so that applications that subsequently
        subscribe to the topic can received the last retained message immediately.
    type: bool
    default: false
  ca_cert:
    type: path
    description:
      - The path to the Certificate Authority certificate files that are to be treated as trusted by this client. If this
        is the only option given then the client operates in a similar manner to a web browser. That is to say it requires
        the broker to have a certificate signed by the Certificate Authorities in ca_certs and communicates using TLS v1,
        but does not attempt any form of authentication. This provides basic network encryption but may not be sufficient
        depending on how the broker is configured.
    aliases: [ca_certs]
  client_cert:
    type: path
    description:
      - The path pointing to the PEM encoded client certificate. If this is set it is used as client information for TLS based
        authentication. Support for this feature is broker dependent.
    aliases: [certfile]
  client_key:
    type: path
    description:
      - The path pointing to the PEM encoded client private key. If this is set it is used as client information for TLS based
        authentication. Support for this feature is broker dependent.
    aliases: [keyfile]
  tls_version:
    description:
      - Specifies the version of the SSL/TLS protocol to be used.
      - By default (if the python version supports it) the highest TLS version is detected. If unavailable, TLS v1 is used.
    type: str
    choices:
      - tlsv1.1
      - tlsv1.2
requirements: [mosquitto]
notes:
  - This module requires a connection to an MQTT broker such as Mosquitto U(http://mosquitto.org) and the I(Paho) C(mqtt)
    Python client (U(https://pypi.org/project/paho-mqtt/)).
author: "Jan-Piet Mens (@jpmens)"
"""

EXAMPLES = r"""
- name: Publish a message on an MQTT topic
  community.general.mqtt:
    topic: 'service/ansible/{{ ansible_hostname }}'
    payload: 'Hello at {{ ansible_date_time.iso8601 }}'
    qos: 0
    retain: false
    client_id: ans001
  delegate_to: localhost
"""

# ===========================================
# MQTT module support methods.
#

import os
import ssl
import traceback

HAS_PAHOMQTT = True
PAHOMQTT_IMP_ERR = None
try:
    import socket

    import paho.mqtt.publish as mqtt
except ImportError:
    PAHOMQTT_IMP_ERR = traceback.format_exc()
    HAS_PAHOMQTT = False

from ansible.module_utils.basic import AnsibleModule, missing_required_lib

# ===========================================
# Main
#


def main():
    # From https://docs.python.org/3/library/ssl.html#constants, this:
    #
    # > Deprecated since version 3.6: OpenSSL has deprecated all version specific protocols. Use the default protocol PROTOCOL_TLS_SERVER or
    # > PROTOCOL_TLS_CLIENT with SSLContext.minimum_version and SSLContext.maximum_version instead.
    #
    # @TODO: update the use of `ssl` constants
    tls_map = {
        "tlsv1.2": ssl.PROTOCOL_TLSv1_2,
        "tlsv1.1": ssl.PROTOCOL_TLSv1_1,
    }

    module = AnsibleModule(
        argument_spec=dict(
            server=dict(default="localhost"),
            port=dict(default=1883, type="int"),
            topic=dict(required=True),
            payload=dict(required=True),
            client_id=dict(),
            qos=dict(default="0", choices=["0", "1", "2"]),
            retain=dict(default=False, type="bool"),
            username=dict(),
            password=dict(no_log=True),
            ca_cert=dict(type="path", aliases=["ca_certs"]),
            client_cert=dict(type="path", aliases=["certfile"]),
            client_key=dict(type="path", aliases=["keyfile"]),
            tls_version=dict(choices=["tlsv1.1", "tlsv1.2"]),
        ),
        supports_check_mode=True,
    )

    if not HAS_PAHOMQTT:
        module.fail_json(msg=missing_required_lib("paho-mqtt"), exception=PAHOMQTT_IMP_ERR)

    server = module.params.get("server", "localhost")
    port = module.params.get("port", 1883)
    topic = module.params.get("topic")
    payload = module.params.get("payload")
    client_id = module.params.get("client_id", "")
    qos = int(module.params.get("qos", 0))
    retain = module.params.get("retain")
    username = module.params.get("username", None)
    password = module.params.get("password", None)
    ca_certs = module.params.get("ca_cert", None)
    certfile = module.params.get("client_cert", None)
    keyfile = module.params.get("client_key", None)
    tls_version = module.params.get("tls_version", None)

    if client_id is None:
        client_id = f"{socket.getfqdn()}_{os.getpid()}"

    if payload and payload == "None":
        payload = None

    auth = None
    if username is not None:
        auth = {"username": username, "password": password}

    tls = None
    if ca_certs is not None:
        if tls_version:
            tls_version = tls_map.get(tls_version, ssl.PROTOCOL_TLS)

        tls = {
            "ca_certs": ca_certs,
            "certfile": certfile,
            "keyfile": keyfile,
            "tls_version": tls_version,
        }

    try:
        mqtt.single(
            topic, payload, qos=qos, retain=retain, client_id=client_id, hostname=server, port=port, auth=auth, tls=tls
        )
    except Exception as e:
        module.fail_json(msg=f"unable to publish to MQTT broker {e}", exception=traceback.format_exc())

    module.exit_json(changed=False, topic=topic)


if __name__ == "__main__":
    main()



# Copyright (c) 2020, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import re

_VALUE_RE = re.compile("^[\x21-\x3a\x3c-\x7e]*$")
_LABEL_RE = re.compile("^[0-9a-zA-Z][0-9a-zA-Z-]*$")


def _check_value(value: str) -> None:
    if not _VALUE_RE.match(value):
        raise ValueError(f"Invalid value {value!r}")


def _check_label(label: str, what: str) -> None:
    if not _LABEL_RE.match(label):
        raise ValueError(f"Invalid {what} {label!r}")


def _check_domain_name(value: str) -> None:
    for p in value.split("."):
        _check_label(p, "label")


def parse_issue_value(
    value: str, *, check_for_duplicates: bool = True, strict: bool = True
) -> tuple[str | None, list[tuple[str, str]]]:
    """
    Given a CAA issue property, parses it according to the syntax defined in RFC 8659.

    More precisely, see https://www.rfc-editor.org/rfc/rfc8659.html#section-4.2.

    If ``check_for_duplicates == True``, duplicate tags are reported as an error.
    If ``strict == True``, invalid characters are reported as an error.
    """
    parts = [v.strip(" \t") for v in value.split(";")]
    if len(parts) > 1 and not parts[-1]:
        del parts[-1]
    domain_name = parts[0] or None
    if domain_name is not None and strict:
        _check_domain_name(domain_name)
    pairs = []
    previous_tags: set[str] = set()
    for part in parts[1:]:
        pieces = part.split("=", 1)
        if len(pieces) != 2:
            raise ValueError(f"{part!r} is not of the form tag=value")
        tag, value = pieces[0].rstrip(" \t"), pieces[1].lstrip(" \t")
        if strict:
            _check_label(tag, "tag")
            _check_value(value)
        pairs.append((tag, value))
        if check_for_duplicates:
            if tag in previous_tags:
                raise ValueError(f"Tag {tag!r} appears multiple times")
            previous_tags.add(tag)
    return domain_name, pairs


def join_issue_value(
    domain_name: str | None,
    pairs: list[tuple[str, str]],
    *,
    check_for_duplicates: bool = True,
    strict: bool = True,
) -> str:
    """
    Given a domain name and a list of tag-value pairs, joins them according
    to the syntax defined in RFC 8659.

    More precisely, see https://www.rfc-editor.org/rfc/rfc8659.html#section-4.2.

    If ``check_for_duplicates == True``, duplicate tags are reported as an error.
    If ``strict == True``, invalid characters are reported as an error.
    """
    if domain_name is not None and strict:
        _check_domain_name(domain_name)
    parts = [domain_name or ""]
    previous_tags: set[str] = set()
    for tag, value in pairs:
        if strict:
            _check_label(tag, "tag")
            _check_value(value)
        parts.append(f"{tag}={value}")
        if check_for_duplicates:
            if tag in previous_tags:
                raise ValueError(f"Tag {tag!r} appears multiple times")
            previous_tags.add(tag)
    return "; ".join(parts)


__all__ = ("parse_issue_value",)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_chassis_profile
short_description: Manage Chassis Profiles in Cisco Intersight
description:
  - Create, update, and delete Chassis Profiles on Cisco Intersight.
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs/chassis/Profile/post/).
  - This module does not manage deployment of the chassis profile.
extends_documentation_fragment: intersight
options:
  state:
    description:
      - If C(present), will verify the resource is present and will create if needed.
      - If C(absent), will verify the resource is absent and will delete if needed.
    type: str
    choices: [present, absent]
    default: present
  organization:
    description:
      - The name of the Organization this resource is assigned to.
      - Profiles and Policies that are created within a Custom Organization are applicable only to devices in the same Organization.
    type: str
    default: default
  name:
    description:
      - The name assigned to the Chassis Profile.
      - The name must be between 1 and 64 alphanumeric characters, allowing special characters :-_.
    type: str
    required: true
  description:
    description:
      - The user-defined description of the Chassis Profile.
      - Description can contain letters(a-z, A-Z), numbers(0-9), hyphen(-), period(.), colon(:), or an underscore(_).
    type: str
    aliases: [descr]
  tags:
    description:
      - List of tags in Key:<user-defined key> Value:<user-defined value> format.
    type: list
    elements: dict
  assigned_chassis_name:
    description:
      - The name of the chassis to assign to this profile.
      - The module resolves the chassis name to a MOID via the C(/equipment/Chasses) API endpoint.
      - If not provided, the profile will be created without a chassis assignment.
    type: str
  imc_access_policy:
    description:
      - Name of IMC Access Policy to associate with this profile.
    type: str
  power_policy:
    description:
      - Name of Power Policy to associate with this profile.
    type: str
  snmp_policy:
    description:
      - Name of SNMP Policy to associate with this profile.
    type: str
  thermal_policy:
    description:
      - Name of Thermal Policy to associate with this profile.
    type: str
author:
  - Ron Gershburg (@rgershbu)
'''

EXAMPLES = r'''
- name: Create a basic Chassis Profile
  cisco.intersight.intersight_chassis_profile:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "Chassis-Profile-01"
    description: "Basic chassis profile"
    tags:
      - Key: "Environment"
        Value: "Production"
    state: present

- name: Create a Chassis Profile with chassis assignment and policies
  cisco.intersight.intersight_chassis_profile:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "Chassis-Profile-02"
    description: "Chassis profile with assignment and policies"
    assigned_chassis_name: "FOX2534P0GG"
    imc_access_policy: "IMC-Access-Prod"
    power_policy: "Power-Grid-Redundancy"
    snmp_policy: "SNMP-Monitoring"
    thermal_policy: "Thermal-Balanced"
    state: present

- name: Delete a Chassis Profile
  cisco.intersight.intersight_chassis_profile:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "Chassis-Profile-01"
    state: absent
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": {
        "Name": "Chassis-Profile-02",
        "ObjectType": "chassis.Profile",
        "AssignedChassis": {
            "Moid": "67e3ebbd61767530012e5a93",
            "ObjectType": "equipment.Chassis"
        },
        "PolicyBucket": [
            {
                "Moid": "69b2af9c6275723101974d23",
                "ObjectType": "access.Policy"
            },
            {
                "Moid": "69b2af9c6275723101974d24",
                "ObjectType": "power.Policy"
            }
        ],
        "Tags": [
            {
                "Key": "Environment",
                "Value": "Production"
            }
        ]
    }
'''


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import (
    IntersightModule,
    intersight_argument_spec,
    resolve_policy_bucket,
    sync_policy_bucket,
)


POLICY_MAPPING = {
    'imc_access_policy': {'resource_path': '/access/Policies', 'object_type': 'access.Policy'},
    'power_policy': {'resource_path': '/power/Policies', 'object_type': 'power.Policy'},
    'snmp_policy': {'resource_path': '/snmp/Policies', 'object_type': 'snmp.Policy'},
    'thermal_policy': {'resource_path': '/thermal/Policies', 'object_type': 'thermal.Policy'},
}


def resolve_assigned_chassis(intersight, chassis_name):
    """Resolve a chassis name to its MOID via the equipment/Chasses endpoint."""
    chassis_moid = intersight.get_moid_by_name(
        resource_path='/equipment/Chasses',
        resource_name=chassis_name,
    )
    if not chassis_moid:
        intersight.module.fail_json(
            msg=f"Chassis '{chassis_name}' not found. Verify the chassis name via GET /equipment/Chasses."
        )
    return chassis_moid


def unassign_chassis_if_needed(intersight, resource_path, profile_name, organization_moid):
    """Unassign the chassis from a profile before deletion.

    The Intersight API forbids deleting a chassis profile while a chassis is
    associated with it. This function checks for an existing assignment and
    PATCHes AssignedChassis to null when one is found.
    """
    filter_str = f"Name eq '{profile_name}' and Organization.Moid eq '{organization_moid}'"
    intersight.get_resource(
        resource_path=resource_path,
        query_params={'$filter': filter_str},
    )

    profile = intersight.result['api_response']
    if not profile.get('Moid'):
        return

    assigned = profile.get('AssignedChassis')
    if assigned and assigned.get('Moid'):
        intersight.configure_resource(
            moid=profile['Moid'],
            resource_path=resource_path,
            body={'AssignedChassis': None},
            query_params={},
        )


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        organization=dict(type='str', default='default'),
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        tags=dict(type='list', elements='dict'),
        assigned_chassis_name=dict(type='str'),
        imc_access_policy=dict(type='str'),
        power_policy=dict(type='str'),
        snmp_policy=dict(type='str'),
        thermal_policy=dict(type='str'),
    )

    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
    )

    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    resource_path = '/chassis/Profiles'

    intersight.api_body = {
        'Organization': {
            'Name': intersight.module.params['organization'],
        },
        'Name': intersight.module.params['name'],
    }

    if intersight.module.params['state'] == 'present':
        intersight.set_tags_and_description()

        chassis_name = intersight.module.params.get('assigned_chassis_name')
        if chassis_name:
            chassis_moid = resolve_assigned_chassis(intersight, chassis_name)
            intersight.api_body['AssignedChassis'] = {
                'Moid': chassis_moid,
                'ObjectType': 'equipment.Chassis',
            }

    if intersight.module.params['state'] == 'absent':
        organization_moid = intersight.get_moid_by_name(
            resource_path='/organization/Organizations',
            resource_name=intersight.module.params['organization'],
        )
        if organization_moid:
            unassign_chassis_if_needed(intersight, resource_path, intersight.module.params['name'], organization_moid)

    intersight.configure_policy_or_profile(resource_path=resource_path)
    profile_response = intersight.result['api_response']
    profile_moid = profile_response.get('Moid') if profile_response else None

    if profile_moid and intersight.module.params['state'] == 'present':
        organization_name = intersight.module.params['organization']
        desired_bucket = resolve_policy_bucket(intersight, organization_name, POLICY_MAPPING)
        current_bucket = profile_response.get('PolicyBucket') or []
        bucket_path = f'/chassis/Profiles/{profile_moid}/PolicyBucket'
        sync_policy_bucket(intersight, bucket_path, desired_bucket, current_bucket)

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2014, Chris Hoffman <choffman@chathamfinancial.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r'''
---
module: win_service
short_description: Manage Windows services
description:
- Manage Windows services.
- For non-Windows targets, use the M(ansible.builtin.service) module instead.
options:
  dependencies:
    description:
    - A list of service dependencies to set for this particular service.
    - This should be a list of service names and not the display name of the
      service.
    - This works by C(dependency_action) to either add/remove or set the
      services in this list.
    type: list
    elements: str
  dependency_action:
    description:
    - Used in conjunction with C(dependency) to either add the dependencies to
      the existing service dependencies.
    - Remove the dependencies to the existing dependencies.
    - Set the dependencies to only the values in the list replacing the
      existing dependencies.
    type: str
    choices: [ add, remove, set ]
    default: set
  desktop_interact:
    description:
    - Whether to allow the service user to interact with the desktop.
    - This can only be set to C(true) when using the C(LocalSystem) username.
    - This can only be set to C(true) when the I(service_type) is
      C(win32_own_process) or C(win32_share_process).
    type: bool
    default: no
  description:
    description:
      - The description to set for the service.
    type: str
  display_name:
    description:
      - The display name to set for the service.
    type: str
  error_control:
    description:
    - The severity of the error and action token if the service fails to start.
    - A new service defaults to C(normal).
    - C(critical) will log the error and restart the system with the last-known
      good configuration. If the startup fails on reboot then the system will
      fail to operate.
    - C(ignore) ignores the error.
    - C(normal) logs the error in the event log but continues.
    - C(severe) is like C(critical) but a failure on the last-known good
      configuration reboot startup will be ignored.
    choices:
    - critical
    - ignore
    - normal
    - severe
    type: str
  failure_actions:
    description:
    - A list of failure actions the service controller should take on each
      failure of a service.
    - The service manager will run the actions from first to last defined until
      the service starts. If I(failure_reset_period_sec) has been exceeded then
      the failure actions will restart from the beginning.
    - If all actions have been performed the the service manager will repeat
      the last service defined.
    - The existing actions will be replaced with the list defined in the task
      if there is a mismatch with any of them.
    - Set to an empty list to delete all failure actions on a service
      otherwise an omitted or null value preserves the existing actions on the
      service.
    type: list
    elements: dict
    suboptions:
      delay_ms:
        description:
        - The time to wait, in milliseconds, before performing the specified action.
        default: 0
        type: raw
        aliases:
        - delay
      type:
        description:
        - The action to be performed.
        - C(none) will perform no action, when used this should only be set as
          the last action.
        - C(reboot) will reboot the host, when used this should only be set as
          the last action as the reboot will reset the action list back to the
          beginning.
        - C(restart) will restart the service.
        - C(run_command) will run the command specified by I(failure_command).
        required: yes
        type: str
        choices:
        - none
        - reboot
        - restart
        - run_command
  failure_actions_on_non_crash_failure:
    description:
    - Controls whether failure actions will be performed on non crash failures
      or not.
    type: bool
  failure_command:
    description:
    - The command to run for a C(run_command) failure action.
    - Set to an empty string to remove the command.
    type: str
  failure_reboot_msg:
    description:
    - The message to be broadcast to users logged on the host for a C(reboot)
      failure action.
    - Set to an empty string to remove the message.
    type: str
  failure_reset_period_sec:
    description:
    - The time in seconds after which the failure action list resets back to
      the start of the list if there are no failures.
    - To set this value, I(failure_actions) must have at least 1 action
      present.
    - Specify C('0xFFFFFFFF') to set an infinite reset period.
    type: raw
    aliases:
    - failure_reset_period
  force_dependent_services:
    description:
    - If C(true), stopping or restarting a service with dependent services will
      force the dependent services to stop or restart also.
    - If C(false), stopping or restarting a service with dependent services may
      fail.
    type: bool
    default: no
  load_order_group:
    description:
    - The name of the load ordering group of which this service is a member.
    - Specify an empty string to remove the existing load order group of a
      service.
    type: str
  name:
    description:
    - Name of the service.
    - If only the name parameter is specified, the module will report
      on whether the service exists or not without making any changes.
    required: yes
    type: str
  path:
    description:
    - The path to the executable to set for the service.
    type: str
  password:
    description:
    - The password to set the service to start as.
    - This and the C(username) argument should be supplied together when using a local or domain account.
    - If omitted then the password will continue to use the existing value password set.
    - If specifying C(LocalSystem), C(NetworkService), C(LocalService), the C(NT SERVICE), or a gMSA this field can be
      omitted as those accounts have no password.
    type: str
  pre_shutdown_timeout_ms:
    description:
    - The time in which the service manager waits after sending a preshutdown
      notification to the service until it proceeds to continue with the other
      shutdown actions.
    aliases:
    - pre_shutdown_timeout
    type: raw
  required_privileges:
    description:
    - A list of privileges the service must have when starting up.
    - When set the service will only have the privileges specified on its
      access token.
    - The I(username) of the service must already have the privileges assigned.
    - The existing privileges will be replace with the list defined in the task
      if there is a mismatch with any of them.
    - Set to an empty list to remove all required privileges, otherwise an
      omitted or null value will keep the existing privileges.
    - See L(privilege text constants,https://docs.microsoft.com/en-us/windows/win32/secauthz/privilege-constants)
      for a list of privilege constants that can be used.
    type: list
    elements: str
  service_type:
    description:
    - The type of service.
    - The default type of a new service is C(win32_own_process).
    - I(desktop_interact) can only be set if the service type is
      C(win32_own_process) or C(win32_share_process).
    choices:
    - user_own_process
    - user_share_process
    - win32_own_process
    - win32_share_process
    type: str
  sid_info:
    description:
    - Used to define the behaviour of the service's access token groups.
    - C(none) will not add any groups to the token.
    - C(restricted) will add the C(NT SERVICE\<service name>) SID to the access
      token's groups and restricted groups.
    - C(unrestricted) will add the C(NT SERVICE\<service name>) SID to the
      access token's groups.
    choices:
    - none
    - restricted
    - unrestricted
    type: str
  start_mode:
    description:
    - Set the startup type for the service.
    - A newly created service will default to C(auto).
    type: str
    choices: [ auto, delayed, disabled, manual ]
  state:
    description:
    - The desired state of the service.
    - C(started)/C(stopped)/C(absent)/C(paused) are idempotent actions that will not run
      commands unless necessary.
    - C(restarted) will always bounce the service.
    - Only services that support the paused state can be paused, you can
      check the return value C(can_pause_and_continue).
    - You can only pause a service that is already started.
    - A newly created service will default to C(stopped).
    type: str
    choices: [ absent, paused, started, stopped, restarted ]
  update_password:
    description:
    - When set to C(always) and I(password) is set, the module will always report a change and set the password.
    - Set to C(on_create) to only set the password if the module needs to create the service.
    - If I(username) was specified and the service changed to that username then I(password) will also be changed if
      specified.
    - The current default is C(on_create) but this behaviour may change in the future, it is best to be explicit here.
    choices:
    - always
    - on_create
    type: str
  username:
    description:
    - The username to set the service to start as.
    - Can also be set to C(LocalSystem) or C(SYSTEM) to use the SYSTEM account.
    - A newly created service will default to C(LocalSystem).
    - If using a custom user account, it must have the C(SeServiceLogonRight)
      granted to be able to start up. You can use the M(ansible.windows.win_user_right) module
      to grant this user right for you.
    - Set to C(NT SERVICE\service name) to run as the NT SERVICE account for that service.
    - This can also be a gMSA in the form C(DOMAIN\gMSA$).
    type: str
notes:
- This module historically returning information about the service in its return values. These should be avoided in
  favour of the M(ansible.windows.win_service_info) module.
- Most of the options in this module are non-driver services that you can view in SCManager. While you can edit driver
  services, not all functionality may be available.
- The user running the module must have the following access rights on the service to be able to use it with this
  module - C(SERVICE_CHANGE_CONFIG), C(SERVICE_ENUMERATE_DEPENDENTS), C(SERVICE_QUERY_CONFIG), C(SERVICE_QUERY_STATUS).
- Changing the state or removing the service will also require futher rights depending on what needs to be done.
seealso:
- module: ansible.builtin.service
- module: community.windows.win_nssm
- module: ansible.windows.win_service_info
- module: ansible.windows.win_user_right
author:
- Chris Hoffman (@chrishoffman)
'''

EXAMPLES = r'''
- name: Restart a service
  ansible.windows.win_service:
    name: spooler
    state: restarted

- name: Set service startup mode to auto and ensure it is started
  ansible.windows.win_service:
    name: spooler
    start_mode: auto
    state: started

- name: Pause a service
  ansible.windows.win_service:
    name: Netlogon
    state: paused

- name: Ensure that WinRM is started when the system has settled
  ansible.windows.win_service:
    name: WinRM
    start_mode: delayed

# A new service will also default to the following values:
# - username: LocalSystem
# - state: stopped
# - start_mode: auto
- name: Create a new service
  ansible.windows.win_service:
    name: service name
    path: C:\temp\test.exe

- name: Create a new service with extra details
  ansible.windows.win_service:
    name: service name
    path: C:\temp\test.exe
    display_name: Service Name
    description: A test service description

- name: Remove a service
  ansible.windows.win_service:
    name: service name
    state: absent

# This is required to be set for non-service accounts that need to run as a service
- name: Grant domain account the SeServiceLogonRight user right
  ansible.windows.win_user_right:
    name: SeServiceLogonRight
    users:
      - DOMAIN\User
    action: add

- name: Set the log on user to a domain account
  ansible.windows.win_service:
    name: service name
    state: restarted
    username: DOMAIN\User
    password: Password

- name: Set the log on user to a local account
  ansible.windows.win_service:
    name: service name
    state: restarted
    username: .\Administrator
    password: Password

- name: Set the log on user to Local System
  ansible.windows.win_service:
    name: service name
    state: restarted
    username: SYSTEM

- name: Set the log on user to Local System and allow it to interact with the desktop
  ansible.windows.win_service:
    name: service name
    state: restarted
    username: SYSTEM
    desktop_interact: true

- name: Set the log on user to Network Service
  ansible.windows.win_service:
    name: service name
    state: restarted
    username: NT AUTHORITY\NetworkService

- name: Set the log on user to Local Service
  ansible.windows.win_service:
    name: service name
    state: restarted
    username: NT AUTHORITY\LocalService

- name: Set the log on user as the services' virtual account
  ansible.windows.win_service:
    name: service name
    username: NT SERVICE\service name

- name: Set the log on user as a gMSA
  ansible.windows.win_service:
    name: service name
    username: DOMAIN\gMSA$  # The end $ is important and should be set for all gMSA

- name: Set dependencies to ones only in the list
  ansible.windows.win_service:
    name: service name
    dependencies: [service1, service2]

- name: Add dependencies to existing dependencies
  ansible.windows.win_service:
    name: service name
    dependencies: [service1, service2]
    dependency_action: add

- name: Remove dependencies from existing dependencies
  ansible.windows.win_service:
    name: service name
    dependencies:
      - service1
      - service2
    dependency_action: remove

- name: Set required privileges for a service
  ansible.windows.win_service:
    name: service name
    username: NT SERVICE\LocalService
    required_privileges:
      - SeBackupPrivilege
      - SeRestorePrivilege

- name: Remove all required privileges for a service
  ansible.windows.win_service:
    name: service name
    username: NT SERVICE\LocalService
    required_privileges: []

- name: Set failure actions for a service with no reset period
  ansible.windows.win_service:
    name: service name
    failure_actions:
      - type: restart
      - type: run_command
        delay_ms: 1000
      - type: restart
        delay_ms: 5000
      - type: reboot
    failure_command: C:\Windows\System32\cmd.exe /c mkdir C:\temp
    failure_reboot_msg: Restarting host because service name has failed
    failure_reset_period_sec: '0xFFFFFFFF'

- name: Set only 1 failure action without a repeat of the last action
  ansible.windows.win_service:
    name: service name
    failure_actions:
      - type: restart
        delay_ms: 5000
      - type: none

- name: Remove failure action information
  ansible.windows.win_service:
    name: service name
    failure_actions: []
    failure_command: ''  # removes the existing command
    failure_reboot_msg: ''  # removes the existing reboot msg
'''

RETURN = r'''
exists:
    description: Whether the service exists or not.
    returned: success
    type: bool
    sample: true
name:
    description: The service name or id of the service.
    returned: success and service exists
    type: str
    sample: CoreMessagingRegistrar
display_name:
    description: The display name of the installed service.
    returned: success and service exists
    type: str
    sample: CoreMessaging
state:
    description: The current running status of the service.
    returned: success and service exists
    type: str
    sample: stopped
start_mode:
    description: The startup type of the service.
    returned: success and service exists
    type: str
    sample: manual
path:
    description: The path to the service executable.
    returned: success and service exists
    type: str
    sample: C:\Windows\system32\svchost.exe -k LocalServiceNoNetwork
can_pause_and_continue:
    description: Whether the service can be paused and unpaused.
    returned: success and service exists
    type: bool
    sample: true
description:
    description: The description of the service.
    returned: success and service exists
    type: str
    sample: Manages communication between system components.
username:
    description: The username that runs the service.
    returned: success and service exists
    type: str
    sample: LocalSystem
desktop_interact:
    description: Whether the current user is allowed to interact with the desktop.
    returned: success and service exists
    type: bool
    sample: false
dependencies:
    description: A list of services that is depended by this service.
    returned: success and service exists
    type: list
    sample: false
depended_by:
    description: A list of services that depend on this service.
    returned: success and service exists
    type: list
    sample: false
'''



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networks_switch_dhcp_server_policy
short_description: Resource module for networks _switch _dhcp _server _policy
description:
  - Manage operation update of the resource networks _switch _dhcp _server _policy.
    - > Update the DHCP server settings. Blocked/allowed servers are only applied
    when default policy is allow/block, respectively.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  alerts:
    description: Alert settings for DHCP servers.
    suboptions:
      email:
        description: Email alert settings for DHCP servers.
        suboptions:
          enabled:
            description: When enabled, send an email if a new DHCP server is seen.
              Default value is false.
            type: bool
        type: dict
    type: dict
  allowedServers:
    description: List the MAC addresses of DHCP servers to permit on the network when
      defaultPolicy is set to block. An empty array will clear the entries.
    elements: str
    type: list
  arpInspection:
    description: Dynamic ARP Inspection settings.
    suboptions:
      enabled:
        description: Enable or disable Dynamic ARP Inspection on the network. Default
          value is false.
        type: bool
    type: dict
  blockedServers:
    description: List the MAC addresses of DHCP servers to block on the network when
      defaultPolicy is set to allow. An empty array will clear the entries.
    elements: str
    type: list
  defaultPolicy:
    description: '''allow'' or ''block'' new DHCP servers. Default value is ''allow''.'
    type: str
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for switch updateNetworkSwitchDhcpServerPolicy
    description: Complete reference of the updateNetworkSwitchDhcpServerPolicy API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-network-switch-dhcp-server-policy
notes:
  - SDK Method used are
    switch.Switch.update_network_switch_dhcp_server_policy,
  - Paths used are
    put /networks/{networkId}/switch/dhcpServerPolicy,
"""

EXAMPLES = r"""
- name: Update all
  cisco.meraki.networks_switch_dhcp_server_policy:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    alerts:
      email:
        enabled: true
    allowedServers:
      - 00:50:56:00:00:01
      - 00:50:56:00:00:02
    alwaysAllowedServers:
      - 00:50:56:00:00:05
      - 00:50:56:00:00:06
    arpInspection:
      enabled: true
    blockedServers:
      - 00:50:56:00:00:03
      - 00:50:56:00:00:04
    defaultPolicy: block
    networkId: string
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "alerts": {
        "email": {
          "enabled": true
        }
      },
      "defaultPolicy": "string",
      "blockedServers": [
        "string"
      ],
      "allowedServers": [
        "string"
      ],
      "alwaysAllowedServers": [
        "string"
      ],
      "arpInspection": {
        "enabled": true,
        "unsupportedModels": [
          "string"
        ]
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "certified"}

DOCUMENTATION = r"""
---
module: aci_tag
short_description: Tagging of ACI objects (tag:Annotation, tag:Inst, and tag:Tag)
description:
- Tagging a object on Cisco ACI fabric.
options:
  dn:
    description:
    - Unique Distinguished Name (DN) from ACI object model.
    type: str
  tag_key:
    description:
    - Unique identifier of tag object.
    type: str
  tag_value:
    description:
    - Value of the property.
    type: str
    default: ""
  tag_type:
    description:
    - Type of tag object.
    type: str
    choices: [ annotation, instance, tag ]
    required: true
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
extends_documentation_fragment:
- cisco.aci.aci

notes:
- The ACI object must exist before using this module in your playbook.
- CAVEAT - Due to deprecation of the 'tagInst' object, creating a tag with tag_type 'instance' automatically generates a
  'annotation' tag_type tag with an empty value. When deleting a tag_type 'instance', the 'tagAnnotation' object will
  remain present and needs to be deleted separately.

seealso:
- name: APIC Management Information Model reference
  description: More information about the internal APIC classes B(tag:Annotation), B(tag:Inst), and B(tag:Tag).
  link: https://developer.cisco.com/docs/apic-mim-ref/
- name: Cisco APIC System Management Configuration Guide
  description: More information about the tagging can be found in the Cisco APIC System Management Configuration Guide.
  link: https://www.cisco.com/c/en/us/support/cloud-systems-management/application-policy-infrastructure-controller-apic/tsd-products-support-series-home.html
author:
- Akini Ross (@akinross)
"""

EXAMPLES = r"""
- name: Add a new annotation tag
  cisco.aci.aci_tag:
    host: apic
    username: admin
    password: SomeSecretPassword
    dn: SomeValidAciDN
    tag_key: foo
    tag_value: bar
    tag_type: annotation
    state: present
  delegate_to: localhost
- name: Delete annotation tag
  cisco.aci.aci_tag:
    host: apic
    username: admin
    password: SomeSecretPassword
    dn: SomeValidAciDN
    tag_key: foo
    tag_type: annotation
    state: absent
  delegate_to: localhost
- name: Query annotation tag
  cisco.aci.aci_tag:
    host: apic
    username: admin
    password: SomeSecretPassword
    dn: SomeValidAciDN
    tag_key: foo
    tag_type: annotation
    state: query
  delegate_to: localhost
- name: Query annotation tags
  cisco.aci.aci_tag:
    host: apic
    username: admin
    password: SomeSecretPassword
    tag_type: annotation
    state: query
  delegate_to: localhost
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(
        dn=dict(type="str"),
        tag_key=dict(type="str", no_log=False),
        tag_value=dict(type="str", default=""),
        tag_type=dict(type="str", choices=["annotation", "instance", "tag"], required=True),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["dn", "tag_key"]],
            ["state", "present", ["dn", "tag_key"]],
        ],
    )

    aci = ACIModule(module)

    dn = module.params.get("dn")
    tag_key = module.params.get("tag_key")
    tag_value = module.params.get("tag_value")
    tag_type = module.params.get("tag_type")
    state = module.params.get("state")

    if module.params.get("dn") is not None:
        dn = dn.lstrip("uni/")

    aci_type = dict(
        annotation=dict(dn="{0}/annotationKey-{1}".format(dn, tag_key), name="tagAnnotation", config=dict(value=tag_value)),
        instance=dict(dn="{0}/tag-{1}".format(dn, tag_key), name="tagInst", config=dict()),
        tag=dict(dn="{0}/tagKey-{1}".format(dn, tag_key), name="tagTag", config=dict(value=tag_value)),
    )

    aci.construct_url(
        root_class=dict(
            aci_class=aci_type[tag_type]["name"],
            aci_rn=aci_type[tag_type]["dn"],
            module_object=tag_key,
            target_filter={"name": tag_key},
        )
    )

    aci.get_existing()

    if state == "present":
        aci.payload(aci_class=aci_type[tag_type]["name"], class_config=aci_type[tag_type]["config"])
        aci.get_diff(aci_class=aci_type[tag_type]["name"])
        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_wireless_devices_radsec_certificates_authorities_info
short_description: Information module for organizations _wireless _devices _radsec
  _certificates _authorities
description:
  - Get all organizations _wireless _devices _radsec _certificates _authorities. -
    > Query for details on the organization's RADSEC device Certificate Authority
    certificates CAs. The primary CA signs all the certificates that devices present
    when establishing a secure connection to RADIUS servers via RADSEC protocol. This
    API returns an array of the status of all of the CAs as well as their contents,
    if they've been generated. An organization will have at most one CA unless the
    CA is being rotated.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  organizationId:
    description:
      - OrganizationId path parameter. Organization ID.
    type: str
  certificateAuthorityIds:
    description:
      - >
        CertificateAuthorityIds query parameter. Optional parameter to filter CAs
        by one or more CA IDs. All returned CAs will have an ID that is an exact match.
    elements: str
    type: list
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for wireless getOrganizationWirelessDevicesRadsecCertificatesAuthorities
    description: Complete reference of the getOrganizationWirelessDevicesRadsecCertificatesAuthorities
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-organization-wireless-devices-radsec-certificates-authorities
notes:
  - SDK Method used are
    wireless.Wireless.get_organization_wireless_devices_radsec_certificates_authorities,
  - Paths used are
    get /organizations/{organizationId}/wireless/devices/radsec/certificates/authorities,
"""

EXAMPLES = r"""
- name: Get all organizations _wireless _devices _radsec _certificates _authorities
  cisco.meraki.organizations_wireless_devices_radsec_certificates_authorities_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    certificateAuthorityIds: []
    organizationId: string
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: list
  elements: dict
  sample: >
    [
      {
        "items": [
          {
            "certificateAuthorityId": "string",
            "status": "string",
            "contents": "string"
          }
        ],
        "meta": {
          "counts": {
            "items": {
              "total": 0,
              "remaining": 0
            }
          }
        }
      }
    ]
"""



#!/usr/bin/python

# Copyright (c) Ansible project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: keycloak_clientscope_rolemappings

short_description: Allows administration of Keycloak clientscope scope mappings to restrict the usage of certain roles to
  specific clientscopes

version_added: 13.1.0

description:
  - This module allows you to add or remove Keycloak roles from clientscopes using the Keycloak REST API. It requires access
    to the REST API using OpenID Connect; the user connecting and the client being used must have the requisite access rights.
    In a default Keycloak installation, C(admin-cli) and an admin user would work, as would a separate client definition with
    the scope tailored to your needs and a user having the expected roles.
  - Attributes are multi-valued in the Keycloak API. All attributes are lists of individual values and are returned that way
    by this module. You may pass single values for attributes when calling the module, and this is translated into a list
    suitable for the API.
attributes:
  check_mode:
    support: full
  diff_mode:
    support: full

options:
  state:
    description:
      - State of the role mapping.
      - On V(present), all roles in O(role_names) are mapped if not exist yet.
      - On V(absent), all roles mapping in O(role_names) are removed if they exist.
    default: 'present'
    type: str
    choices:
      - present
      - absent

  realm:
    type: str
    description:
      - The Keycloak realm under which clients resides.
    default: 'master'

  clientscope_id:
    required: true
    type: str
    description:
      - Roles provided in O(role_names) will be added to this clientscope.

  client_id:
    type: str
    description:
      - If the O(role_names) are client roles, the client ID under which it resides.
      - If this parameter is absent, the roles are considered realm roles.

  role_names:
    required: true
    type: list
    elements: str
    description:
      - Names of roles to add.
      - If O(client_id) is present, all roles must be under this client.
      - If O(client_id) is absent, all roles must be under the realm.

extends_documentation_fragment:
  - community.general._keycloak
  - community.general._keycloak.actiongroup_keycloak
  - community.general._attributes

author:
  - Felix Grzelka (@felix-grzelka)
  # This module was adapted from keycloak_client_rolescope, which was written by Andre Desrosiers (@desand01).
"""

EXAMPLES = r"""
- name: Add roles to clientscope
  community.general.keycloak_clientscope_rolemappings:
    auth_keycloak_url: https://auth.example.com/auth
    auth_realm: master
    auth_username: USERNAME
    auth_password: PASSWORD
    realm: MyCustomRealm
    client_id: frontend-client-public
    clientscope_id: frontend-clientscope
    role_names:
      - backend-role-admin
      - backend-role-user

- name: Remove roles from clientscope
  community.general.keycloak_clientscope_rolemappings:
    auth_keycloak_url: https://auth.example.com/auth
    auth_realm: master
    auth_username: USERNAME
    auth_password: PASSWORD
    realm: MyCustomRealm
    client_id: frontend-client-public
    clientscope_id: frontend-clientscope
    role_names:
      - backend-role-admin
    state: absent

- name: Add realm roles to clientscope
  community.general.keycloak_clientscope_rolemappings:
    auth_keycloak_url: https://auth.example.com/auth
    auth_realm: master
    auth_username: USERNAME
    auth_password: PASSWORD
    realm: MyCustomRealm
    clientscope_id: frontend-clientscope
    role_names:
      - realm-role-admin
      - realm-role-user
"""

RETURN = r"""
end_state:
  description: Representation of clientscope scope mappings after module execution.
  returned: on success
  type: list
  elements: dict
  sample:
    [
      {
        "clientRole": false,
        "composite": false,
        "containerId": "77f9bd4e-13a6-451e-9c72-ee6997299c1f",
        "description": "User role",
        "id": "9e155ef7-86f5-4def-b507-581ce7b87013",
        "name": "realm-role-user"
      },
      {
        "clientRole": false,
        "composite": false,
        "containerId": "77f9bd4e-13a6-451e-9c72-ee6997299c1f",
        "description": "Admin role",
        "id": "9e155ef7-86f5-4def-b507-581ce7b87013",
        "name": "realm-role-admin"
      }
    ]
"""

import copy

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._keycloak import (
    KeycloakAPI,
    KeycloakError,
    get_token,
    keycloak_argument_spec,
)


def main():
    argument_spec = keycloak_argument_spec()

    meta_args = dict(
        client_id=dict(type="str"),
        clientscope_id=dict(type="str", required=True),
        realm=dict(type="str", default="master"),
        role_names=dict(type="list", elements="str", required=True),
        state=dict(type="str", default="present", choices=["present", "absent"]),
    )

    argument_spec.update(meta_args)

    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)

    result = dict(changed=False, msg="", diff={}, end_state={})

    # Obtain access token, initialize API
    try:
        connection_header = get_token(module.params)
    except KeycloakError as e:
        module.fail_json(msg=str(e))

    kc = KeycloakAPI(module, connection_header)

    realm = module.params["realm"]
    client_id = module.params["client_id"]
    clientscope_id = module.params["clientscope_id"]
    role_names = module.params["role_names"]
    state = module.params["state"]

    realm_object = kc.get_realm_by_id(realm)
    if not realm_object:
        module.fail_json(msg=f"Failed to retrieve realm '{realm}'")

    clientscope_object = kc.get_clientscope_by_name(clientscope_id, realm)
    if not clientscope_object:
        module.fail_json(msg=f"Failed to retrieve client-scope '{clientscope_id}'")

    if client_id:
        # add client role
        client_object = kc.get_client_by_clientid(client_id, realm)
        if not client_object:
            module.fail_json(msg=f"Failed to retrieve client '{realm}.{client_id}'")
        if client_object["fullScopeAllowed"] and state == "present":
            module.fail_json(msg=f"FullScopeAllowed is active for Client '{realm}.{client_id}'")

        before_roles = kc.get_clientscope_scope_mappings_client(clientscope_object["id"], client_object["id"], realm)
        available_roles_by_name = kc.get_client_roles_by_id(client_object["id"], realm)
    else:
        # add realm role
        before_roles = kc.get_clientscope_scope_mappings_realm(clientscope_object["id"], realm)
        available_roles_by_name = kc.get_realm_roles(realm)

    # convert to indexed Dict by name
    available_roles_by_name = {role["name"]: role for role in available_roles_by_name}
    before_roles_by_name = {role["name"]: role for role in before_roles}
    desired_roles = copy.deepcopy(before_roles)
    changed_roles = []

    if state == "present":
        # update desired
        for role_name in role_names:
            if role_name not in available_roles_by_name:
                if client_id:
                    module.fail_json(msg=f"Failed to retrieve role '{realm}.{client_id}.{role_name}'")
                else:
                    module.fail_json(msg=f"Failed to retrieve role '{realm}.{role_name}'")
            if role_name not in before_roles_by_name:
                changed_roles.append(available_roles_by_name[role_name])
                desired_roles.append(available_roles_by_name[role_name])
    else:
        # remove role if present
        for role_name in role_names:
            if role_name in before_roles_by_name:
                changed_roles.append(before_roles_by_name[role_name])
                desired_roles.remove(available_roles_by_name[role_name])

    before_roles = sorted(before_roles, key=lambda d: d["name"])
    desired_role_mapping = sorted(desired_roles, key=lambda d: d["name"])

    result["changed"] = bool(changed_roles)

    if module._diff:
        result["diff"] = dict(before={"roles": before_roles}, after={"roles": desired_role_mapping})

    if not result["changed"]:
        # no changes
        result["end_state"] = before_roles
        result["msg"] = f"No changes required for clientscope {clientscope_id}."
    elif state == "present":
        # doing update
        if module.check_mode:
            result["end_state"] = desired_role_mapping
        elif client_id:
            result["end_state"] = kc.update_clientscope_scope_mappings_client(
                changed_roles, clientscope_object["id"], client_object["id"], realm
            )
        else:
            result["end_state"] = kc.update_clientscope_scope_mappings_realm(
                changed_roles, clientscope_object["id"], realm
            )
        result["msg"] = f"Clientscope scope mappings for {clientscope_id} have been updated"
    else:
        # doing delete
        if module.check_mode:
            result["end_state"] = desired_role_mapping
        elif client_id:
            result["end_state"] = kc.delete_clientscope_scope_mappings_client(
                changed_roles, clientscope_object["id"], client_object["id"], realm
            )
        else:
            result["end_state"] = kc.delete_clientscope_scope_mappings_realm(
                changed_roles, clientscope_object["id"], realm
            )
        result["msg"] = f"Clientscope scope mappings for {clientscope_id} have been deleted"
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_assurance_alerts_overview_info
short_description: Information module for organizations _assurance _alerts _overview
description:
  - Get all organizations _assurance _alerts _overview.
  - Return overview of active health alerts for an organization.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  organizationId:
    description:
      - OrganizationId path parameter. Organization ID.
    type: str
  networkId:
    description:
      - NetworkId query parameter. Optional parameter to filter alerts overview by
        network ids.
    type: str
  severity:
    description:
      - Severity query parameter. Optional parameter to filter alerts overview by
        severity type.
    type: str
  types:
    description:
      - Types query parameter. Optional parameter to filter by alert type.
    elements: str
    type: list
  tsStart:
    description:
      - TsStart query parameter. Optional parameter to filter by starting timestamp.
    type: str
  tsEnd:
    description:
      - TsEnd query parameter. Optional parameter to filter by end timestamp.
    type: str
  category:
    description:
      - Category query parameter. Optional parameter to filter by category.
    type: str
  serials:
    description:
      - Serials query parameter. Optional parameter to filter by primary device serial.
    elements: str
    type: list
  deviceTypes:
    description:
      - DeviceTypes query parameter. Optional parameter to filter by device types.
    elements: str
    type: list
  deviceTags:
    description:
      - DeviceTags query parameter. Optional parameter to filter by device tags.
    elements: str
    type: list
  active:
    description:
      - Active query parameter. Optional parameter to filter by active alerts defaults
        to true.
    type: bool
  dismissed:
    description:
      - Dismissed query parameter. Optional parameter to filter by dismissed alerts
        defaults to false.
    type: bool
  resolved:
    description:
      - Resolved query parameter. Optional parameter to filter by resolved alerts
        defaults to false.
    type: bool
  suppressAlertsForOfflineNodes:
    description:
      - >
        SuppressAlertsForOfflineNodes query parameter. When set to true the api will
        only return connectivity alerts for a given device if that device is in an
        offline state. This only applies to devices. This is ignored when resolved
        is true. Example If a Switch has a VLan Mismatch and is Unreachable. Only
        the Unreachable alert will be returned. Defaults to false.
    type: bool
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for organizations getOrganizationAssuranceAlertsOverview
    description: Complete reference of the getOrganizationAssuranceAlertsOverview
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-organization-assurance-alerts-overview
notes:
  - SDK Method used are
    organizations.Organizations.get_organization_assurance_alerts_overview,
  - Paths used are
    get /organizations/{organizationId}/assurance/alerts/overview,
"""

EXAMPLES = r"""
- name: Get all organizations _assurance _alerts _overview
  cisco.meraki.organizations_assurance_alerts_overview_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    networkId: string
    severity: string
    types: []
    tsStart: string
    tsEnd: string
    category: string
    serials: []
    deviceTypes: []
    deviceTags: []
    active: true
    dismissed: true
    resolved: true
    suppressAlertsForOfflineNodes: true
    organizationId: string
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "total": 0,
      "bySeverity": [
        {
          "type": "string",
          "count": 0
        }
      ]
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2021, Manuel Widmer <mawidmer@cisco.com>
# Copyright: (c) 2021, Anvitha Jain (@anvitha-jain) <anvjain@cisco.com>
# Copyright: (c) 2023, Akini Ross (@akinross) <akinross@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: aci_vmm_controller
short_description: Manage VMM Controller for virtual domains profiles (vmm:CtrlrP)
description:
- Manage vCenter virtual domains on Cisco ACI fabrics.
options:
  name:
    description:
    - Name of VMM Controller.
    type: str
    aliases: []
  controller_hostname:
    description:
    - Hostname or IP of the controller.
    type: str
    aliases: []
  dvs_version:
    description:
    - Version of the VMware DVS.
    type: str
    aliases: []
    choices: [ 'unmanaged', '5.1', '5.5', '6.0', '6.5', '6.6', '7.0', '8.0' ]
  stats_collection:
    description:
    - Whether stats collection is enabled.
    type: str
    choices: [ 'enabled', 'disabled' ]
    default: disabled
    aliases: []
  domain:
    description:
    - Name of the virtual domain profile.
    type: str
    aliases: [ domain_name, domain_profile ]
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
  credentials:
    description:
    - Name of the VMM credentials to be used
    type: str
  inband_management_epg:
    description:
    - Name of the management EPG to be used by the controller. Only supports in-band management EPGs for now.
    type: str
  name_alias:
    description:
    - The alias for the current object. This relates to the nameAlias field in ACI.
    type: str
  datacenter:
    description:
    - Name of the data center, as seen in vCenter
    type: str
  vm_provider:
    description:
    - The VM platform for VMM Domains.
    - Support for Kubernetes was added in ACI v3.0.
    - Support for CloudFoundry, OpenShift and Red Hat was added in ACI v3.1.
    type: str
    choices: [ cloudfoundry, kubernetes, microsoft, openshift, openstack, redhat, vmware, nutanix ]
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation

seealso:
- module: cisco.aci.aci_domain
- module: cisco.aci.aci_vmm_credential
- name: APIC Management Information Model reference
  description: More information about the internal APIC classes B(vmm:DomP)
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Manuel Widmer (@lumean)
- Anvitha Jain (@anvitha-jain)
- Akini Ross (@akinross)
"""

EXAMPLES = r"""
- name: Add controller to VMware VMM domain
  cisco.aci.aci_vmm_controller:
    host: apic
    username: admin
    password: SomeSecretPassword
    domain: vmware_dom
    name: vCenter1
    controller_hostname: 10.1.1.1
    dvs_version: unmanaged
    vm_provider: vmware
    credentials: vCenterCredentials1
    datacenter: DC1
    state: present

- name: Remove controller from VMware VMM domain
  cisco.aci.aci_vmm_controller:
    host: apic
    username: admin
    password: SomeSecretPassword
    domain: vmware_dom
    name: vCenter1
    vm_provider: vmware
    state: absent

- name: Query a specific VMware VMM controller
  cisco.aci.aci_vmm_controller:
    host: apic
    username: admin
    password: SomeSecretPassword
    domain: vmware_dom
    name: vCenter1
    vm_provider: vmware
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all VMware VMM controller
  cisco.aci.aci_vmm_controller:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost
  register: query_result
"""


RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec
from ansible_collections.cisco.aci.plugins.module_utils.constants import VM_PROVIDER_MAPPING, VM_SCOPE_MAPPING


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(
        name=dict(type="str"),
        controller_hostname=dict(type="str"),
        dvs_version=dict(type="str", choices=["unmanaged", "5.1", "5.5", "6.0", "6.5", "6.6", "7.0", "8.0"]),
        stats_collection=dict(type="str", default="disabled", choices=["enabled", "disabled"]),
        domain=dict(type="str", aliases=["domain_name", "domain_profile"]),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
        credentials=dict(type="str"),
        inband_management_epg=dict(type="str"),
        name_alias=dict(type="str"),
        datacenter=dict(type="str"),
        vm_provider=dict(type="str", choices=list(VM_PROVIDER_MAPPING)),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["domain", "vm_provider", "name"]],
            ["state", "present", ["domain", "vm_provider", "name"]],
        ],
    )

    name = module.params.get("name")
    controller_hostname = module.params.get("controller_hostname")
    dvs_version = module.params.get("dvs_version")
    stats_collection = module.params.get("stats_collection")
    domain = module.params.get("domain")
    state = module.params.get("state")
    credentials = module.params.get("credentials")
    inband_management_epg = module.params.get("inband_management_epg")
    name_alias = module.params.get("name_alias")
    datacenter = module.params.get("datacenter")
    vm_provider = module.params.get("vm_provider")

    controller_class = "vmmCtrlrP"

    aci = ACIModule(module)

    child_classes = ["vmmRsMgmtEPg", "vmmRsAcc"]

    aci.construct_url(
        root_class=dict(
            aci_class="vmmProvP",
            aci_rn="vmmp-{0}".format(VM_PROVIDER_MAPPING.get(vm_provider)),
            module_object=vm_provider,
            target_filter={"name": vm_provider},
        ),
        subclass_1=dict(
            aci_class="vmmDomP",
            aci_rn="dom-{0}".format(domain),
            module_object=domain,
            target_filter={"name": domain},
        ),
        subclass_2=dict(
            aci_class="vmmCtrlrP",
            aci_rn="ctrlr-{0}".format(name),
            module_object=name,
            target_filter={"name": name},
        ),
        child_classes=child_classes,
    )

    # vmmProvP is not allowed to execute a query with rsp-subtree set in the filter_string
    # due to complicated url construction logic which should be refactored creating a temporary fix inside module
    # TODO refactor url construction logic if more occurences of rsp-subtree not supported problem appear
    # check if the url is pointing towards the vmmProvP class and rsp-subtree is set in the filter_string
    if aci.url.split("/")[-1].startswith("vmmp-") and "rsp-subtree" in aci.filter_string:
        if name:
            aci.url = "{0}/api/class/vmmCtrlrP.json".format(aci.base_url)
            aci.filter_string = '?query-target-filter=eq(vmmCtrlrP.name,"{0}")&rsp-subtree=full&rsp-subtree-class={1}'.format(name, ",".join(child_classes))
        else:
            aci.url = "{0}/api/mo/uni/vmmp-{1}.json".format(aci.base_url, VM_PROVIDER_MAPPING.get(vm_provider))
            aci.filter_string = ""

    aci.get_existing()

    if state == "present":
        children = list()
        if inband_management_epg is not None:
            children.append(dict(vmmRsMgmtEPg=dict(attributes=dict(tDn="uni/tn-mgmt/mgmtp-default/inb-{0}".format(inband_management_epg)))))

        if credentials is not None:
            children.append(
                dict(vmmRsAcc=dict(attributes=dict(tDn="uni/vmmp-{0}/dom-{1}/usracc-{2}".format(VM_PROVIDER_MAPPING.get(vm_provider), domain, credentials))))
            )

        aci.payload(
            aci_class=controller_class,
            class_config=dict(
                name=name,
                hostOrIp=controller_hostname,
                dvsVersion=dvs_version,
                statsMode=stats_collection,
                rootContName=datacenter,
                nameAlias=name_alias,
                scope=VM_SCOPE_MAPPING.get(vm_provider),
            ),
            child_configs=children,
        )

        aci.get_diff(aci_class=controller_class)

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2019, Guillaume Martinez (lunik@tiwabbit.fr)
# Copyright (c) 2015, Werner Dijkerman (ikben@werner-dijkerman.nl)
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: gitlab_project
short_description: Creates/updates/deletes GitLab Projects
description:
  - When the project does not exist in GitLab, it is created.
  - When the project does exist and O(state=absent), the project is deleted.
  - When changes are made to the project, the project is updated.
author:
  - Werner Dijkerman (@dj-wasabi)
  - Guillaume Martinez (@Lunik)
requirements:
  - python-gitlab python module
extends_documentation_fragment:
  - community.general._auth_basic
  - community.general._gitlab
  - community.general._attributes

attributes:
  check_mode:
    support: full
  diff_mode:
    support: none

options:
  allow_merge_on_skipped_pipeline:
    description:
      - Allow merge when skipped pipelines exist.
    type: bool
    version_added: "3.4.0"
  avatar_path:
    description:
      - Absolute path image to configure avatar. File size should not exceed 200 kb.
      - This option is only used on creation, not for updates.
    type: path
    version_added: "4.2.0"
  build_timeout:
    description:
      - Maximum number of seconds a CI job can run.
      - If not specified on creation, GitLab imposes a default value.
    type: int
    version_added: "10.6.0"
  builds_access_level:
    description:
      - V(private) means that repository CI/CD is allowed only to project members.
      - V(disabled) means that repository CI/CD is disabled.
      - V(enabled) means that repository CI/CD is enabled.
    type: str
    choices: ["private", "disabled", "enabled"]
    version_added: "6.2.0"
  ci_config_path:
    description:
      - Custom path to the CI configuration file for this project.
    type: str
    version_added: "3.7.0"
  container_expiration_policy:
    description:
      - Project cleanup policy for its container registry.
    type: dict
    suboptions:
      cadence:
        description:
          - How often cleanup should be run.
        type: str
        choices: ["1d", "7d", "14d", "1month", "3month"]
      enabled:
        description:
          - Enable the cleanup policy.
        type: bool
      keep_n:
        description:
          - Number of tags kept per image name.
          - V(0) clears the field.
        type: int
        choices: [0, 1, 5, 10, 25, 50, 100]
      older_than:
        description:
          - Destroy tags older than this.
          - V(0d) clears the field.
        type: str
        choices: ["0d", "7d", "14d", "30d", "90d"]
      name_regex:
        description:
          - Destroy tags matching this regular expression.
        type: str
      name_regex_keep:
        description:
          - Keep tags matching this regular expression.
        type: str
    version_added: "9.3.0"
  container_registry_access_level:
    description:
      - V(private) means that container registry is allowed only to project members.
      - V(disabled) means that container registry is disabled.
      - V(enabled) means that container registry is enabled.
    type: str
    choices: ["private", "disabled", "enabled"]
    version_added: "6.2.0"
  default_branch:
    description:
      - The default branch name for this project.
      - For project creation, this option requires O(initialize_with_readme=true).
      - For project update, the branch must exist.
      - Supports project's default branch update since community.general 8.0.0.
    type: str
    version_added: "4.2.0"
  description:
    description:
      - An description for the project.
    type: str
  environments_access_level:
    description:
      - V(private) means that deployment to environment is allowed only to project members.
      - V(disabled) means that deployment to environment is disabled.
      - V(enabled) means that deployment to environment is enabled.
    type: str
    choices: ["private", "disabled", "enabled"]
    version_added: "6.4.0"
  feature_flags_access_level:
    description:
      - V(private) means that feature rollout is allowed only to project members.
      - V(disabled) means that feature rollout is disabled.
      - V(enabled) means that feature rollout is enabled.
    type: str
    choices: ["private", "disabled", "enabled"]
    version_added: "6.4.0"
  forking_access_level:
    description:
      - V(private) means that repository forks is allowed only to project members.
      - V(disabled) means that repository forks are disabled.
      - V(enabled) means that repository forks are enabled.
    type: str
    choices: ["private", "disabled", "enabled"]
    version_added: "6.2.0"
  group:
    description:
      - ID or the full path of the group of which this projects belongs to.
    type: str
  import_url:
    description:
      - Git repository which is imported into gitlab.
      - GitLab server needs read access to this git repository.
    type: str
  infrastructure_access_level:
    description:
      - V(private) means that configuring infrastructure is allowed only to project members.
      - V(disabled) means that configuring infrastructure is disabled.
      - V(enabled) means that configuring infrastructure is enabled.
    type: str
    choices: ["private", "disabled", "enabled"]
    version_added: "6.4.0"
  initialize_with_readme:
    description:
      - Initializes the project with a default C(README.md).
      - Is only used when the project is created, and ignored otherwise.
    type: bool
    default: false
    version_added: "4.0.0"
  issues_access_level:
    description:
      - V(private) means that accessing issues tab is allowed only to project members.
      - V(disabled) means that accessing issues tab is disabled.
      - V(enabled) means that accessing issues tab is enabled.
      - O(issues_access_level) and O(issues_enabled) are mutually exclusive.
    type: str
    choices: ["private", "disabled", "enabled"]
    version_added: "9.4.0"
  issues_enabled:
    description:
      - Whether you want to create issues or not.
      - O(issues_access_level) and O(issues_enabled) are mutually exclusive.
    type: bool
    default: true
  lfs_enabled:
    description:
      - Enable Git large file systems to manages large files such as audio, video, and graphics files.
    type: bool
    default: false
    version_added: "2.0.0"
  merge_method:
    description:
      - What requirements are placed upon merges.
      - Possible values are V(merge), V(rebase_merge) merge commit with semi-linear history, V(ff) fast-forward merges only.
    type: str
    choices: ["ff", "merge", "rebase_merge"]
    default: merge
    version_added: "1.0.0"
  merge_requests_enabled:
    description:
      - If merge requests can be made or not.
    type: bool
    default: true
  model_registry_access_level:
    description:
      - V(private) means that accessing model registry tab is allowed only to project members.
      - V(disabled) means that accessing model registry tab is disabled.
      - V(enabled) means that accessing model registry tab is enabled.
    type: str
    choices: ["private", "disabled", "enabled"]
    version_added: "9.3.0"
  monitor_access_level:
    description:
      - V(private) means that monitoring health is allowed only to project members.
      - V(disabled) means that monitoring health is disabled.
      - V(enabled) means that monitoring health is enabled.
    type: str
    choices: ["private", "disabled", "enabled"]
    version_added: "6.4.0"
  name:
    description:
      - The name of the project.
    required: true
    type: str
  only_allow_merge_if_all_discussions_are_resolved:
    description:
      - All discussions on a merge request (MR) have to be resolved.
    type: bool
    version_added: "3.4.0"
  only_allow_merge_if_pipeline_succeeds:
    description:
      - Only allow merges if pipeline succeeded.
    type: bool
    version_added: "3.4.0"
  packages_enabled:
    description:
      - Enable GitLab package repository.
    type: bool
    version_added: "3.4.0"
  pages_access_level:
    description:
      - V(private) means that accessing pages tab is allowed only to project members.
      - V(disabled) means that accessing pages tab is disabled.
      - V(enabled) means that accessing pages tab is enabled.
    type: str
    choices: ["private", "disabled", "enabled"]
    version_added: "9.3.0"
  path:
    description:
      - The path of the project you want to create, this is server_url/O(group)/O(path).
      - If not supplied, O(name) is used.
    type: str
  releases_access_level:
    description:
      - V(private) means that accessing release is allowed only to project members.
      - V(disabled) means that accessing release is disabled.
      - V(enabled) means that accessing release is enabled.
    type: str
    choices: ["private", "disabled", "enabled"]
    version_added: "6.4.0"
  remove_source_branch_after_merge:
    description:
      - Remove the source branch after merge.
    type: bool
    version_added: "3.4.0"
  repository_access_level:
    description:
      - V(private) means that accessing repository is allowed only to project members.
      - V(disabled) means that accessing repository is disabled.
      - V(enabled) means that accessing repository is enabled.
    type: str
    choices: ["private", "disabled", "enabled"]
    version_added: "9.3.0"
  security_and_compliance_access_level:
    description:
      - V(private) means that accessing security and complicance tab is allowed only to project members.
      - V(disabled) means that accessing security and complicance tab is disabled.
      - V(enabled) means that accessing security and complicance tab is enabled.
    type: str
    choices: ["private", "disabled", "enabled"]
    version_added: "6.4.0"
  service_desk_enabled:
    description:
      - Enable Service Desk.
    type: bool
    version_added: "9.3.0"
  shared_runners_enabled:
    description:
      - Enable shared runners for this project.
    type: bool
    version_added: "3.7.0"
  snippets_enabled:
    description:
      - If creating snippets should be available or not.
    type: bool
    default: true
  squash_option:
    description:
      - Squash commits when merging.
    type: str
    choices: ["never", "always", "default_off", "default_on"]
    version_added: "3.4.0"
  state:
    description:
      - Create or delete project.
      - Possible values are present and absent.
    default: present
    type: str
    choices: ["present", "absent"]
  topics:
    description:
      - A topic or list of topics to be assigned to a project.
      - It is compatible with old GitLab server releases (versions before 14, correspond to C(tag_list)).
    type: list
    elements: str
    version_added: "6.6.0"
  username:
    description:
      - Used to create a personal project under a user's name.
    type: str
    version_added: "3.3.0"
  visibility:
    description:
      - V(private) Project access must be granted explicitly for each user.
      - V(internal) The project can be cloned by any logged in user.
      - V(public) The project can be cloned without any authentication.
    default: private
    type: str
    choices: ["private", "internal", "public"]
    aliases:
      - visibility_level
  wiki_enabled:
    description:
      - If an wiki for this project should be available or not.
    type: bool
    default: true
"""

EXAMPLES = r"""
- name: Create GitLab Project
  community.general.gitlab_project:
    api_url: https://gitlab.example.com/
    api_token: "{{ api_token }}"
    name: my_first_project
    group: "10481470"

- name: Delete GitLab Project
  community.general.gitlab_project:
    api_url: https://gitlab.example.com/
    api_token: "{{ access_token }}"
    name: my_first_project
    state: absent
  delegate_to: localhost

- name: Create GitLab Project in group Ansible
  community.general.gitlab_project:
    api_url: https://gitlab.example.com/
    validate_certs: true
    api_username: dj-wasabi
    api_password: "MySecretPassword"
    name: my_first_project
    group: ansible
    issues_enabled: false
    merge_method: rebase_merge
    wiki_enabled: true
    snippets_enabled: true
    import_url: http://git.example.com/example/lab.git
    initialize_with_readme: true
    state: present
  delegate_to: localhost

- name: get the initial root password
  ansible.builtin.shell: |
    grep 'Password:' /etc/gitlab/initial_root_password | sed -e 's/Password\: \(.*\)/\1/'
  register: initial_root_password

- name: Create a GitLab Project using a username/password via oauth_token
  community.general.gitlab_project:
    api_url: https://gitlab.example.com/
    api_username: root
    api_password: "{{ initial_root_password }}"
    name: my_second_project
    group: "10481470"
"""

RETURN = r"""
msg:
  description: Success or failure message.
  returned: always
  type: str
  sample: "Success"

result:
  description: JSON-parsed response from the server.
  returned: always
  type: dict

error:
  description: The error message returned by the GitLab API.
  returned: failed
  type: str
  sample: "400: path is already in use"

project:
  description: API object.
  returned: always
  type: dict
"""


from ansible.module_utils.api import basic_auth_argument_spec
from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._gitlab import (
    auth_argument_spec,
    find_group,
    find_project,
    gitlab,
    gitlab_authentication,
)
from ansible_collections.community.general.plugins.module_utils._version import LooseVersion


class GitLabProject:
    def __init__(self, module, gitlab_instance):
        self._module = module
        self._gitlab = gitlab_instance
        self.project_object = None

    """
    @param project_name Name of the project
    @param namespace Namespace Object (User or Group)
    @param options Options of the project
    """

    def create_or_update_project(self, module, project_name, namespace, options):
        changed = False
        project_options = {
            "allow_merge_on_skipped_pipeline": options["allow_merge_on_skipped_pipeline"],
            "builds_access_level": options["builds_access_level"],
            "build_timeout": options["build_timeout"],
            "ci_config_path": options["ci_config_path"],
            "container_expiration_policy": options["container_expiration_policy"],
            "container_registry_access_level": options["container_registry_access_level"],
            "description": options["description"],
            "environments_access_level": options["environments_access_level"],
            "feature_flags_access_level": options["feature_flags_access_level"],
            "forking_access_level": options["forking_access_level"],
            "infrastructure_access_level": options["infrastructure_access_level"],
            "issues_access_level": options["issues_access_level"],
            "issues_enabled": options["issues_enabled"],
            "lfs_enabled": options["lfs_enabled"],
            "merge_method": options["merge_method"],
            "merge_requests_enabled": options["merge_requests_enabled"],
            "model_registry_access_level": options["model_registry_access_level"],
            "monitor_access_level": options["monitor_access_level"],
            "name": project_name,
            "only_allow_merge_if_all_discussions_are_resolved": options[
                "only_allow_merge_if_all_discussions_are_resolved"
            ],
            "only_allow_merge_if_pipeline_succeeds": options["only_allow_merge_if_pipeline_succeeds"],
            "packages_enabled": options["packages_enabled"],
            "pages_access_level": options["pages_access_level"],
            "releases_access_level": options["releases_access_level"],
            "remove_source_branch_after_merge": options["remove_source_branch_after_merge"],
            "repository_access_level": options["repository_access_level"],
            "security_and_compliance_access_level": options["security_and_compliance_access_level"],
            "service_desk_enabled": options["service_desk_enabled"],
            "shared_runners_enabled": options["shared_runners_enabled"],
            "snippets_enabled": options["snippets_enabled"],
            "squash_option": options["squash_option"],
            "visibility": options["visibility"],
            "wiki_enabled": options["wiki_enabled"],
        }

        # topics was introduced on gitlab >=14 and replace tag_list. We get current gitlab version
        # and check if less than 14. If yes we use tag_list instead topics
        if LooseVersion(self._gitlab.version()[0]) < LooseVersion("14"):
            project_options["tag_list"] = options["topics"]
        else:
            project_options["topics"] = options["topics"]

        # Because we have already call userExists in main()
        if self.project_object is None:
            if options["default_branch"] and not options["initialize_with_readme"]:
                module.fail_json(msg="Param default_branch needs param initialize_with_readme set to true")
            project_options.update(
                {
                    "path": options["path"],
                    "import_url": options["import_url"],
                }
            )
            if options["initialize_with_readme"]:
                project_options["initialize_with_readme"] = options["initialize_with_readme"]
                if options["default_branch"]:
                    project_options["default_branch"] = options["default_branch"]

            project_options = self.get_options_with_value(project_options)
            project = self.create_project(namespace, project_options)

            # add avatar to project
            if options["avatar_path"]:
                try:
                    project.avatar = open(options["avatar_path"], "rb")
                except OSError as e:
                    self._module.fail_json(msg=f"Cannot open {options['avatar_path']}: {e}")

            changed = True
        else:
            if options["default_branch"]:
                project_options["default_branch"] = options["default_branch"]
            changed, project = self.update_project(self.project_object, project_options)

        self.project_object = project
        if changed:
            if self._module.check_mode:
                self._module.exit_json(changed=True, msg=f"Successfully created or updated the project {project_name}")

            try:
                project.save()
            except Exception as e:
                self._module.fail_json(msg=f"Failed to update project: {e} ")
            return True
        return False

    """
    @param namespace Namespace Object (User or Group)
    @param arguments Attributes of the project
    """

    def create_project(self, namespace, arguments):
        if self._module.check_mode:
            return True

        arguments["namespace_id"] = namespace.id
        if "container_expiration_policy" in arguments:
            arguments["container_expiration_policy_attributes"] = arguments["container_expiration_policy"]
        try:
            project = self._gitlab.projects.create(arguments)
        except gitlab.exceptions.GitlabCreateError as e:
            self._module.fail_json(msg=f"Failed to create project: {e} ")

        return project

    """
    @param arguments Attributes of the project
    """

    def get_options_with_value(self, arguments):
        ret_arguments = {k: v for k, v in arguments.items() if v is not None}
        return ret_arguments

    """
    @param project Project Object
    @param arguments Attributes of the project
    """

    def update_project(self, project, arguments):
        changed = False

        for arg_key, arg_value in arguments.items():
            if arg_value is not None:
                if getattr(project, arg_key, None) != arg_value:
                    if arg_key == "container_expiration_policy":
                        old_val = getattr(project, arg_key, {})
                        final_val = {key: value for key, value in arg_value.items() if value is not None}

                        if final_val.get("older_than") == "0d":
                            final_val["older_than"] = None
                        if final_val.get("keep_n") == 0:
                            final_val["keep_n"] = None

                        if all(old_val.get(key) == value for key, value in final_val.items()):
                            continue
                        project.container_expiration_policy_attributes = final_val
                    else:
                        setattr(project, arg_key, arg_value)
                    changed = True

        return (changed, project)

    def delete_project(self):
        if self._module.check_mode:
            return True

        project = self.project_object

        return project.delete()

    """
    @param namespace User/Group object
    @param name Name of the project
    """

    def exists_project(self, namespace, path):
        # When project exists, object will be stored in self.project_object.
        project = find_project(self._gitlab, f"{namespace.full_path}/{path}")
        if project:
            self.project_object = project
            return True
        return False


def main():
    argument_spec = basic_auth_argument_spec()
    argument_spec.update(auth_argument_spec())
    argument_spec.update(
        dict(
            allow_merge_on_skipped_pipeline=dict(type="bool"),
            avatar_path=dict(type="path"),
            builds_access_level=dict(type="str", choices=["private", "disabled", "enabled"]),
            build_timeout=dict(type="int"),
            ci_config_path=dict(type="str"),
            container_expiration_policy=dict(
                type="dict",
                options=dict(
                    cadence=dict(type="str", choices=["1d", "7d", "14d", "1month", "3month"]),
                    enabled=dict(type="bool"),
                    keep_n=dict(type="int", choices=[0, 1, 5, 10, 25, 50, 100]),
                    older_than=dict(type="str", choices=["0d", "7d", "14d", "30d", "90d"]),
                    name_regex=dict(type="str"),
                    name_regex_keep=dict(type="str"),
                ),
            ),
            container_registry_access_level=dict(type="str", choices=["private", "disabled", "enabled"]),
            default_branch=dict(type="str"),
            description=dict(type="str"),
            environments_access_level=dict(type="str", choices=["private", "disabled", "enabled"]),
            feature_flags_access_level=dict(type="str", choices=["private", "disabled", "enabled"]),
            forking_access_level=dict(type="str", choices=["private", "disabled", "enabled"]),
            group=dict(type="str"),
            import_url=dict(type="str"),
            infrastructure_access_level=dict(type="str", choices=["private", "disabled", "enabled"]),
            initialize_with_readme=dict(type="bool", default=False),
            issues_access_level=dict(type="str", choices=["private", "disabled", "enabled"]),
            issues_enabled=dict(type="bool", default=True),
            lfs_enabled=dict(default=False, type="bool"),
            merge_method=dict(type="str", default="merge", choices=["merge", "rebase_merge", "ff"]),
            merge_requests_enabled=dict(type="bool", default=True),
            model_registry_access_level=dict(type="str", choices=["private", "disabled", "enabled"]),
            monitor_access_level=dict(type="str", choices=["private", "disabled", "enabled"]),
            name=dict(type="str", required=True),
            only_allow_merge_if_all_discussions_are_resolved=dict(type="bool"),
            only_allow_merge_if_pipeline_succeeds=dict(type="bool"),
            packages_enabled=dict(type="bool"),
            pages_access_level=dict(type="str", choices=["private", "disabled", "enabled"]),
            path=dict(type="str"),
            releases_access_level=dict(type="str", choices=["private", "disabled", "enabled"]),
            remove_source_branch_after_merge=dict(type="bool"),
            repository_access_level=dict(type="str", choices=["private", "disabled", "enabled"]),
            security_and_compliance_access_level=dict(type="str", choices=["private", "disabled", "enabled"]),
            service_desk_enabled=dict(type="bool"),
            shared_runners_enabled=dict(type="bool"),
            snippets_enabled=dict(default=True, type="bool"),
            squash_option=dict(type="str", choices=["never", "always", "default_off", "default_on"]),
            state=dict(type="str", default="present", choices=["absent", "present"]),
            topics=dict(type="list", elements="str"),
            username=dict(type="str"),
            visibility=dict(
                type="str", default="private", choices=["internal", "private", "public"], aliases=["visibility_level"]
            ),
            wiki_enabled=dict(type="bool", default=True),
        )
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        mutually_exclusive=[
            ["api_username", "api_token"],
            ["api_username", "api_oauth_token"],
            ["api_username", "api_job_token"],
            ["api_token", "api_oauth_token"],
            ["api_token", "api_job_token"],
            ["group", "username"],
            ["issues_access_level", "issues_enabled"],
        ],
        required_together=[
            ["api_username", "api_password"],
        ],
        required_one_of=[["api_username", "api_token", "api_oauth_token", "api_job_token"]],
        supports_check_mode=True,
    )

    # check prerequisites and connect to gitlab server
    gitlab_instance = gitlab_authentication(module)

    allow_merge_on_skipped_pipeline = module.params["allow_merge_on_skipped_pipeline"]
    avatar_path = module.params["avatar_path"]
    builds_access_level = module.params["builds_access_level"]
    build_timeout = module.params["build_timeout"]
    ci_config_path = module.params["ci_config_path"]
    container_expiration_policy = module.params["container_expiration_policy"]
    container_registry_access_level = module.params["container_registry_access_level"]
    default_branch = module.params["default_branch"]
    environments_access_level = module.params["environments_access_level"]
    feature_flags_access_level = module.params["feature_flags_access_level"]
    forking_access_level = module.params["forking_access_level"]
    group_identifier = module.params["group"]
    import_url = module.params["import_url"]
    infrastructure_access_level = module.params["infrastructure_access_level"]
    initialize_with_readme = module.params["initialize_with_readme"]
    issues_access_level = module.params["issues_access_level"]
    issues_enabled = module.params["issues_enabled"]
    lfs_enabled = module.params["lfs_enabled"]
    merge_method = module.params["merge_method"]
    merge_requests_enabled = module.params["merge_requests_enabled"]
    model_registry_access_level = module.params["model_registry_access_level"]
    monitor_access_level = module.params["monitor_access_level"]
    only_allow_merge_if_all_discussions_are_resolved = module.params["only_allow_merge_if_all_discussions_are_resolved"]
    only_allow_merge_if_pipeline_succeeds = module.params["only_allow_merge_if_pipeline_succeeds"]
    packages_enabled = module.params["packages_enabled"]
    pages_access_level = module.params["pages_access_level"]
    project_description = module.params["description"]
    project_name = module.params["name"]
    project_path = module.params["path"]
    releases_access_level = module.params["releases_access_level"]
    remove_source_branch_after_merge = module.params["remove_source_branch_after_merge"]
    repository_access_level = module.params["repository_access_level"]
    security_and_compliance_access_level = module.params["security_and_compliance_access_level"]
    service_desk_enabled = module.params["service_desk_enabled"]
    shared_runners_enabled = module.params["shared_runners_enabled"]
    snippets_enabled = module.params["snippets_enabled"]
    squash_option = module.params["squash_option"]
    state = module.params["state"]
    topics = module.params["topics"]
    username = module.params["username"]
    visibility = module.params["visibility"]
    wiki_enabled = module.params["wiki_enabled"]

    # Set project_path to project_name if it is empty.
    if project_path is None:
        project_path = project_name.replace(" ", "_")

    gitlab_project = GitLabProject(module, gitlab_instance)

    namespace = None
    namespace_id = None
    if group_identifier:
        group = find_group(gitlab_instance, group_identifier)
        if group is None:
            module.fail_json(msg=f"Failed to create project: group {group_identifier} doesn't exist")

        namespace_id = group.id
    else:
        if username:
            namespace = gitlab_instance.namespaces.list(search=username, all=False)[0]
        else:
            namespace = gitlab_instance.namespaces.list(search=gitlab_instance.user.username, all=False)[0]
        namespace_id = namespace.id

    if not namespace_id:
        module.fail_json(msg="Failed to find the namespace or group ID which is required to look up the namespace")

    try:
        namespace = gitlab_instance.namespaces.get(namespace_id)
    except gitlab.exceptions.GitlabGetError as e:
        module.fail_json(msg=f"Failed to find the namespace for the given user: {e}")

    if not namespace:
        module.fail_json(msg="Failed to find the namespace for the project")
    project_exists = gitlab_project.exists_project(namespace, project_path)

    if state == "absent":
        if project_exists:
            gitlab_project.delete_project()
            module.exit_json(changed=True, msg=f"Successfully deleted project {project_name}")
        module.exit_json(changed=False, msg="Project deleted or does not exist")

    if state == "present":
        if gitlab_project.create_or_update_project(
            module,
            project_name,
            namespace,
            {
                "allow_merge_on_skipped_pipeline": allow_merge_on_skipped_pipeline,
                "avatar_path": avatar_path,
                "builds_access_level": builds_access_level,
                "build_timeout": build_timeout,
                "ci_config_path": ci_config_path,
                "container_expiration_policy": container_expiration_policy,
                "container_registry_access_level": container_registry_access_level,
                "default_branch": default_branch,
                "description": project_description,
                "environments_access_level": environments_access_level,
                "feature_flags_access_level": feature_flags_access_level,
                "forking_access_level": forking_access_level,
                "import_url": import_url,
                "infrastructure_access_level": infrastructure_access_level,
                "initialize_with_readme": initialize_with_readme,
                "issues_access_level": issues_access_level,
                "issues_enabled": issues_enabled,
                "lfs_enabled": lfs_enabled,
                "merge_method": merge_method,
                "merge_requests_enabled": merge_requests_enabled,
                "model_registry_access_level": model_registry_access_level,
                "monitor_access_level": monitor_access_level,
                "only_allow_merge_if_all_discussions_are_resolved": only_allow_merge_if_all_discussions_are_resolved,
                "only_allow_merge_if_pipeline_succeeds": only_allow_merge_if_pipeline_succeeds,
                "packages_enabled": packages_enabled,
                "pages_access_level": pages_access_level,
                "path": project_path,
                "releases_access_level": releases_access_level,
                "remove_source_branch_after_merge": remove_source_branch_after_merge,
                "repository_access_level": repository_access_level,
                "security_and_compliance_access_level": security_and_compliance_access_level,
                "service_desk_enabled": service_desk_enabled,
                "shared_runners_enabled": shared_runners_enabled,
                "snippets_enabled": snippets_enabled,
                "squash_option": squash_option,
                "topics": topics,
                "visibility": visibility,
                "wiki_enabled": wiki_enabled,
            },
        ):
            module.exit_json(
                changed=True,
                msg=f"Successfully created or updated the project {project_name}",
                project=gitlab_project.project_object._attrs,
            )
        module.exit_json(
            changed=False,
            msg=f"No need to update the project {project_name}",
            project=gitlab_project.project_object._attrs,
        )


if __name__ == "__main__":
    main()
